#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import glob
from PyQt5 import QtCore, QtGui, QtWidgets
from cryio import parparser, cbfimage
from crymon import reconstruct
from .ui.ui_wdarth import Ui_WDarth
from . import pyqt2bool


class WorkerSignals(QtCore.QObject):
    errorSignal = QtCore.pyqtSignal(str)
    progressSignal = QtCore.pyqtSignal(str)
    stopSignal = QtCore.pyqtSignal()
    finishedSignal = QtCore.pyqtSignal(object)
    nFilesSignal = QtCore.pyqtSignal(int)


class Worker(QtCore.QRunnable):
    def __init__(self, params):
        QtCore.QRunnable.__init__(self)
        self.stop = False
        folder = params['folder']
        self.folder = folder[:-1] if folder[-1] == '/' or folder[-1] == '\\' else folder
        self.params = params
        self.signals = WorkerSignals()
        self.signals.stopSignal.connect(self.stopIt)

    def stopIt(self):
        self.stop = True

    def parfile(self):
        self.par = parparser.ParParser(self.params['par'])

    def makeSlice(self):
        s = reconstruct.Slice()
        s.downsample = self.params['downsample']
        s.lorentz = self.params['lorentz']
        s.scale = self.params['scale']
        if 'layer' in self.params:
            l = self.params['layer']
            s.p0.h, s.p0.k, s.p0.l = [float(i) for i in l['hkl1'].split()]
            s.p1.h, s.p1.k, s.p1.l = [float(i) for i in l['hkl2'].split()]
            s.pc.h, s.pc.k, s.pc.l = [float(i) for i in l['center'].split()]
            s.qmax = float(l['qmax'])
            s.thickness = float(l['thickness'])
            self.reconst = reconstruct.Layer(self.par.parstruct, s)
        else:
            s.p0.h, s.p0.k, s.p0.l = 1, 0, 0
            s.p1.h, s.p1.k, s.p1.l = 0, 1, 0
            s.pc.h, s.pc.k, s.pc.l = 0, 0, 0
            s.dQ = self.params['dQ']
            s.x = self.params['x']
            s.y = self.params['y']
            s.z = self.params['z']
            self.reconst = reconstruct.Volume(self.par.parstruct, s)

    def makemap(self):
        for i, cbf in enumerate(self.cbflist, 1):
            if self.stop:
                break
            image = cbfimage.CbfImage(cbf)
            self.reconst.add(image.array, image.header['Start_angle'], image.header['Angle_increment'])
            self.signals.progressSignal.emit('Reading files')
        self.signals.progressSignal.emit('Calculating CCP4')
        if not self.stop:
            self.reconst.save(self.mapname)
        self.signals.progressSignal.emit('Finished')

    def genMapName(self):
        layer_dirs = os.path.join(self.folder, 'layers')
        try:
            os.mkdir(layer_dirs)
        except OSError:
            if not os.path.isdir(layer_dirs):
                self.stopIt()
                return
        if 'layer' in self.params:
            l = self.params['layer']
            fn = '{}__{}__{}__{}.cbf'.format(self.basename, l['hkl1'], l['hkl2'], l['center']).replace(' ', '_')
            self.mapname = os.path.join(layer_dirs, fn)
        else:
            fn = '{}.ccp4'.format(self.basename)
            self.mapname = os.path.join(layer_dirs, fn)

    def run(self):
        self.basename = os.path.basename(self.folder)
        self.genMapName()
        self.cbflist = glob.glob(os.path.join(self.folder, '*.cbf'))
        self.signals.nFilesSignal.emit(len(self.cbflist))
        self.cbflist.sort()
        self.parfile()
        self.makeSlice()
        self.makemap()
        self.stopIt()
        self.signals.finishedSignal.emit(self)


class WorkerPool(QtCore.QObject):
    progressSignal = QtCore.pyqtSignal(str, int)
    finishedSignal = QtCore.pyqtSignal(list)

    def __init__(self, parent=None):
        QtCore.QObject.__init__(self, parent)
        self.pool = QtCore.QThreadPool.globalInstance()
        self.workers, self.errors = [], []

    def run(self, params):
        folder = params['folder']
        self.files = self.done = 0
        self.workers, self.errors = [], []
        for dirpath, dirnames, filenames in os.walk(folder):
            pars = glob.glob(os.path.join(dirpath, '*_cracker.par'))
            if not pars:
                continue
            layers = params.pop('layers')
            if params['volume']:
                p = params.copy()
                p['par'] = pars[0]
                p['folder'] = dirpath
                self.startWorker(p)
            for layer in layers:
                p = params.copy()
                p['par'] = pars[0]
                p['folder'] = dirpath
                p['layer'] = layer
                self.startWorker(p)

    def startWorker(self, p):
        worker = Worker(p)
        worker.signals.errorSignal.connect(self.workerError)
        worker.signals.progressSignal.connect(self.workerProgress)
        worker.signals.finishedSignal.connect(self.workerFinished)
        worker.signals.nFilesSignal.connect(self.addNFiles)
        self.workers.append(worker)
        self.pool.start(worker)

    def addNFiles(self, n):
        self.files += n

    def stopIt(self):
        for worker in self.workers:
            worker.signals.stopSignal.emit()

    def workerProgress(self, info):
        self.done += 1
        done = 100.0 * self.done / self.files
        self.progressSignal.emit(info, done)

    def workerError(self, folder):
        self.errors.append(folder)

    def workerFinished(self, worker):
        self.workers.remove(worker)
        if not self.workers:
            self.progressSignal.emit('Finished', 100)
            self.finishedSignal.emit(self.errors)


class WDarth(QtWidgets.QDialog, Ui_WDarth):
    stopWorkerSignal = QtCore.pyqtSignal()
    runWorkerSignal = QtCore.pyqtSignal(dict)

    def __init__(self, parent=None):
        QtWidgets.QDialog.__init__(self, parent)
        self._parent = parent
        self.setUI()
        self.setPool()
        self.connectSignals()
        self.workerThread.start()

    def connectSignals(self):
        self.stopWorkerSignal.connect(self.pool.stopIt)
        self.runWorkerSignal.connect(self.pool.run)

    def setPool(self):
        self.workerThread = QtCore.QThread()
        self.pool = WorkerPool()
        self.pool.moveToThread(self.workerThread)
        self.pool.progressSignal.connect(self.showProgress)
        self.pool.finishedSignal.connect(self.workerFinished)

    def setUI(self):
        self.setupUi(self)
        intValidator = QtGui.QIntValidator()
        intValidator.setBottom(1)
        floatValidator = QtGui.QDoubleValidator()
        floatValidator.setBottom(1e-3)
        self.xLineEdit.setValidator(intValidator)
        self.yLineEdit.setValidator(intValidator)
        self.zLineEdit.setValidator(intValidator)
        self.editDownsample.setValidator(intValidator)
        self.editScale.setValidator(intValidator)
        self.dQLineEdit.setValidator(floatValidator)
        self.stopButton.setVisible(False)
        self.loadSettings()

    def showError(self, errors):
        # noinspection PyCallByClass,PyArgumentList,PyTypeChecker
        QtWidgets.QMessageBox.critical(self,
                                       'Darth Vader error',
                                       'Datasets in folders\n{}\nare corrupted: it seems you have aborted the '
                                       'experiments because the headers are empty.\n It cannot be '
                                       'processed.'.format('\n'.join(errors)))

    def showProgress(self, info, value):
        self.runProgressBar.setFormat('{0}: %p%'.format(info))
        self.runProgressBar.setValue(value)

    def closeEvent(self, event):
        self.saveSettings()
        self.stopWorkerSignal.emit()
        self.hide()

    def keyPressEvent(self, event):
        key = event.key()
        if key == QtCore.Qt.Key_Escape:
            pass
        else:
            QtWidgets.QDialog.keyPressEvent(self, event)

    def saveSettings(self):
        s = QtCore.QSettings()
        s.setValue('WDarth/Geometry', self.saveGeometry())
        s.setValue('WDarth/lastFolder', self.lastFolder)
        s.setValue('WDarth/folder', self.folderLineEdit.text())
        s.setValue('WDarth/x', self.xLineEdit.text())
        s.setValue('WDarth/y', self.yLineEdit.text())
        s.setValue('WDarth/z', self.zLineEdit.text())
        s.setValue('WDarth/dQ', self.dQLineEdit.text())
        s.setValue('WDarth/lorentz', self.lorentzCheckBox.isChecked())
        s.setValue('WDarth/treeLayersState', self.treeLayers.header().saveState())
        s.setValue('WDarth/downsample', self.editDownsample.text())
        s.setValue('WDarth/scale', self.editScale.text())
        s.setValue('WDarth/volume', self.volumeCheckBox.isChecked())

    def loadSettings(self):
        s = QtCore.QSettings()
        self.restoreGeometry(s.value('WDarth/Geometry', QtCore.QByteArray()))
        self.lastFolder = s.value('WDarth/lastFolder', u'')
        self.folderLineEdit.setText(s.value('WDarth/folder', u''))
        self.xLineEdit.setText(s.value('WDarth/x', '256'))
        self.yLineEdit.setText(s.value('WDarth/y', '256'))
        self.zLineEdit.setText(s.value('WDarth/z', '256'))
        self.dQLineEdit.setText(s.value('WDarth/dQ', '0.6'))
        self.lorentzCheckBox.setChecked(pyqt2bool(s.value('WDarth/lorentz', False)))
        self.treeLayers.header().restoreState(s.value('WDarth/treeLayersState', QtCore.QByteArray()))
        self.editDownsample.setText(s.value('WDarth/downsample', '1'))
        self.editScale.setText(s.value('WDarth/scale', '1'))
        self.volumeCheckBox.setChecked(pyqt2bool(s.value('WDarth/volume', True)))

    @QtCore.pyqtSlot()
    def on_runButton_clicked(self):
        folder = self.folderLineEdit.text()
        if not folder or not os.path.exists(folder):
            return
        self.runButton.setVisible(False)
        self.stopButton.setVisible(True)
        params = {
            'folder': folder,
            'x': int(self.xLineEdit.text()),
            'y': int(self.yLineEdit.text()),
            'z': int(self.zLineEdit.text()),
            'dQ': float(self.dQLineEdit.text()),
            'lorentz': int(self.lorentzCheckBox.isChecked()),
            'scale': int(self.editScale.text()),
            'volume': self.volumeCheckBox.isChecked(),
            'layers': self.treeLayers.getLayers(),
            'downsample': int(self.editDownsample.text()),
        }
        self.runWorkerSignal.emit(params)

    def workerFinished(self, errors):
        self.runButton.setVisible(True)
        self.stopButton.setVisible(False)
        if errors:
            self.showError(errors)

    @QtCore.pyqtSlot()
    def on_stopButton_clicked(self):
        self.stopWorkerSignal.emit()

    @QtCore.pyqtSlot()
    def on_folderButton_clicked(self):
        # noinspection PyCallByClass,PyTypeChecker
        folder = QtWidgets.QFileDialog.getExistingDirectory(self, 'Select directory', self.lastFolder)
        if not folder:
            return
        self.folderLineEdit.setText(folder)
        self.lastFolder = folder



# -*- coding: utf-8 -*-


from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_WGunnar(object):
    def setupUi(self, WGunnar):
        WGunnar.setObjectName("WGunnar")
        WGunnar.resize(532, 778)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/roerik"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        WGunnar.setWindowIcon(icon)
        WGunnar.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(WGunnar)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_14.setObjectName("horizontalLayout_14")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_12 = QtWidgets.QLabel(WGunnar)
        self.label_12.setObjectName("label_12")
        self.horizontalLayout.addWidget(self.label_12)
        self.deltaEdit = QtWidgets.QLineEdit(WGunnar)
        self.deltaEdit.setObjectName("deltaEdit")
        self.horizontalLayout.addWidget(self.deltaEdit)
        self.horizontalLayout_14.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_6 = QtWidgets.QLabel(WGunnar)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_2.addWidget(self.label_6)
        self.d0Edit = QtWidgets.QLineEdit(WGunnar)
        self.d0Edit.setObjectName("d0Edit")
        self.horizontalLayout_2.addWidget(self.d0Edit)
        self.horizontalLayout_14.addLayout(self.horizontalLayout_2)
        self.verticalLayout_2.addLayout(self.horizontalLayout_14)
        self.horizontalLayout_15 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_15.setObjectName("horizontalLayout_15")
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        self.label = QtWidgets.QLabel(WGunnar)
        self.label.setObjectName("label")
        self.horizontalLayout_12.addWidget(self.label)
        self.lambdaEdit = QtWidgets.QLineEdit(WGunnar)
        self.lambdaEdit.setObjectName("lambdaEdit")
        self.horizontalLayout_12.addWidget(self.lambdaEdit)
        self.horizontalLayout_15.addLayout(self.horizontalLayout_12)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_7 = QtWidgets.QLabel(WGunnar)
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_3.addWidget(self.label_7)
        self.x0Edit = QtWidgets.QLineEdit(WGunnar)
        self.x0Edit.setObjectName("x0Edit")
        self.horizontalLayout_3.addWidget(self.x0Edit)
        self.horizontalLayout_15.addLayout(self.horizontalLayout_3)
        self.verticalLayout_2.addLayout(self.horizontalLayout_15)
        self.horizontalLayout_16 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_16.setObjectName("horizontalLayout_16")
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.label_2 = QtWidgets.QLabel(WGunnar)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_11.addWidget(self.label_2)
        self.alphaEdit = QtWidgets.QLineEdit(WGunnar)
        self.alphaEdit.setObjectName("alphaEdit")
        self.horizontalLayout_11.addWidget(self.alphaEdit)
        self.horizontalLayout_16.addLayout(self.horizontalLayout_11)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_8 = QtWidgets.QLabel(WGunnar)
        self.label_8.setObjectName("label_8")
        self.horizontalLayout_4.addWidget(self.label_8)
        self.y0Edit = QtWidgets.QLineEdit(WGunnar)
        self.y0Edit.setObjectName("y0Edit")
        self.horizontalLayout_4.addWidget(self.y0Edit)
        self.horizontalLayout_16.addLayout(self.horizontalLayout_4)
        self.verticalLayout_2.addLayout(self.horizontalLayout_16)
        self.horizontalLayout_17 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_17.setObjectName("horizontalLayout_17")
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.label_3 = QtWidgets.QLabel(WGunnar)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_10.addWidget(self.label_3)
        self.betaEdit = QtWidgets.QLineEdit(WGunnar)
        self.betaEdit.setObjectName("betaEdit")
        self.horizontalLayout_10.addWidget(self.betaEdit)
        self.horizontalLayout_17.addLayout(self.horizontalLayout_10)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_9 = QtWidgets.QLabel(WGunnar)
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_5.addWidget(self.label_9)
        self.omegaEdit = QtWidgets.QLineEdit(WGunnar)
        self.omegaEdit.setObjectName("omegaEdit")
        self.horizontalLayout_5.addWidget(self.omegaEdit)
        self.horizontalLayout_17.addLayout(self.horizontalLayout_5)
        self.verticalLayout_2.addLayout(self.horizontalLayout_17)
        self.horizontalLayout_18 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_18.setObjectName("horizontalLayout_18")
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.label_4 = QtWidgets.QLabel(WGunnar)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_9.addWidget(self.label_4)
        self.phixEdit = QtWidgets.QLineEdit(WGunnar)
        self.phixEdit.setObjectName("phixEdit")
        self.horizontalLayout_9.addWidget(self.phixEdit)
        self.horizontalLayout_18.addLayout(self.horizontalLayout_9)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.label_10 = QtWidgets.QLabel(WGunnar)
        self.label_10.setObjectName("label_10")
        self.horizontalLayout_6.addWidget(self.label_10)
        self.kappaEdit = QtWidgets.QLineEdit(WGunnar)
        self.kappaEdit.setObjectName("kappaEdit")
        self.horizontalLayout_6.addWidget(self.kappaEdit)
        self.horizontalLayout_18.addLayout(self.horizontalLayout_6)
        self.verticalLayout_2.addLayout(self.horizontalLayout_18)
        self.horizontalLayout_19 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_19.setObjectName("horizontalLayout_19")
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.label_5 = QtWidgets.QLabel(WGunnar)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_8.addWidget(self.label_5)
        self.phiyEdit = QtWidgets.QLineEdit(WGunnar)
        self.phiyEdit.setObjectName("phiyEdit")
        self.horizontalLayout_8.addWidget(self.phiyEdit)
        self.horizontalLayout_19.addLayout(self.horizontalLayout_8)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.label_11 = QtWidgets.QLabel(WGunnar)
        self.label_11.setObjectName("label_11")
        self.horizontalLayout_7.addWidget(self.label_11)
        self.phiEdit = QtWidgets.QLineEdit(WGunnar)
        self.phiEdit.setObjectName("phiEdit")
        self.horizontalLayout_7.addWidget(self.phiEdit)
        self.horizontalLayout_19.addLayout(self.horizontalLayout_7)
        self.verticalLayout_2.addLayout(self.horizontalLayout_19)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setSizeConstraint(QtWidgets.QLayout.SetMinimumSize)
        self.verticalLayout.setObjectName("verticalLayout")
        self.label_13 = QtWidgets.QLabel(WGunnar)
        self.label_13.setObjectName("label_13")
        self.verticalLayout.addWidget(self.label_13)
        self.ubmText = QtWidgets.QPlainTextEdit(WGunnar)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.ubmText.sizePolicy().hasHeightForWidth())
        self.ubmText.setSizePolicy(sizePolicy)
        self.ubmText.setMaximumBlockCount(0)
        self.ubmText.setObjectName("ubmText")
        self.verticalLayout.addWidget(self.ubmText)
        self.verticalLayout_2.addLayout(self.verticalLayout)
        self.resTable = QtWidgets.QTableWidget(WGunnar)
        self.resTable.setObjectName("resTable")
        self.resTable.setColumnCount(3)
        self.resTable.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.resTable.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.resTable.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.resTable.setHorizontalHeaderItem(2, item)
        self.verticalLayout_2.addWidget(self.resTable)
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_13.setObjectName("horizontalLayout_13")
        self.parButton = QtWidgets.QPushButton(WGunnar)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/open"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.parButton.setIcon(icon1)
        self.parButton.setObjectName("parButton")
        self.horizontalLayout_13.addWidget(self.parButton)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_13.addItem(spacerItem)
        self.exitButton = QtWidgets.QPushButton(WGunnar)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/exit"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.exitButton.setIcon(icon2)
        self.exitButton.setObjectName("exitButton")
        self.horizontalLayout_13.addWidget(self.exitButton)
        self.verticalLayout_2.addLayout(self.horizontalLayout_13)

        self.retranslateUi(WGunnar)
        QtCore.QMetaObject.connectSlotsByName(WGunnar)

    def retranslateUi(self, WGunnar):
        _translate = QtCore.QCoreApplication.translate
        WGunnar.setWindowTitle(_translate("WGunnar", "Hrœrekr Multiple Beam"))
        self.label_12.setText(_translate("WGunnar", "<html><head/><body><p>&Delta;</p></body></html>"))
        self.label_6.setText(_translate("WGunnar", "<html><head/><body><p>d<sub>0</sub></p></body></html>"))
        self.label.setText(_translate("WGunnar", "<html><head/><body><p>&lambda;</p></body></html>"))
        self.label_7.setText(_translate("WGunnar", "<html><head/><body><p>x<sub>0</sub></p></body></html>"))
        self.label_2.setText(_translate("WGunnar", "<html><head/><body><p>&alpha;</p></body></html>"))
        self.label_8.setText(_translate("WGunnar", "<html><head/><body><p>y<sub>0</sub></p></body></html>"))
        self.label_3.setText(_translate("WGunnar", "<html><head/><body><p>&beta;</p></body></html>"))
        self.label_9.setText(_translate("WGunnar", "<html><head/><body><p>&omega;</p></body></html>"))
        self.label_4.setText(_translate("WGunnar", "<html><head/><body><p>&phi;<sub>x</sub></p></body></html>"))
        self.label_10.setText(_translate("WGunnar", "<html><head/><body><p>&kappa;</p></body></html>"))
        self.label_5.setText(_translate("WGunnar", "<html><head/><body><p>&phi;<sub>y</sub></p></body></html>"))
        self.label_11.setText(_translate("WGunnar", "<html><head/><body><p>&phi;</p></body></html>"))
        self.label_13.setText(_translate("WGunnar", "UB matrix"))
        item = self.resTable.horizontalHeaderItem(0)
        item.setText(_translate("WGunnar", "(h, k, l)"))
        item = self.resTable.horizontalHeaderItem(1)
        item.setText(_translate("WGunnar", "(x, y)"))
        item = self.resTable.horizontalHeaderItem(2)
        item.setText(_translate("WGunnar", "ah"))
        self.parButton.setText(_translate("WGunnar", "Open .par file"))
        self.exitButton.setText(_translate("WGunnar", "Exit"))

from . import resources_rc



# -*- coding: utf-8 -*-


from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_WSleuth(object):
    def setupUi(self, WSleuth):
        WSleuth.setObjectName("WSleuth")
        WSleuth.resize(970, 530)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/sleuth"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        WSleuth.setWindowIcon(icon)
        WSleuth.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout(WSleuth)
        self.horizontalLayout_14.setObjectName("horizontalLayout_14")
        self.groupBox = QtWidgets.QGroupBox(WSleuth)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox.sizePolicy().hasHeightForWidth())
        self.groupBox.setSizePolicy(sizePolicy)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(self.groupBox)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.x1SpinBox = QtWidgets.QSpinBox(self.groupBox)
        self.x1SpinBox.setMaximum(65536)
        self.x1SpinBox.setObjectName("x1SpinBox")
        self.horizontalLayout.addWidget(self.x1SpinBox)
        self.horizontalLayout_6.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_2 = QtWidgets.QLabel(self.groupBox)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_2.addWidget(self.label_2)
        self.y1SpinBox = QtWidgets.QSpinBox(self.groupBox)
        self.y1SpinBox.setMaximum(65536)
        self.y1SpinBox.setObjectName("y1SpinBox")
        self.horizontalLayout_2.addWidget(self.y1SpinBox)
        self.horizontalLayout_6.addLayout(self.horizontalLayout_2)
        self.verticalLayout.addLayout(self.horizontalLayout_6)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_3 = QtWidgets.QLabel(self.groupBox)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_3.addWidget(self.label_3)
        self.x2SpinBox = QtWidgets.QSpinBox(self.groupBox)
        self.x2SpinBox.setMaximum(65536)
        self.x2SpinBox.setObjectName("x2SpinBox")
        self.horizontalLayout_3.addWidget(self.x2SpinBox)
        self.horizontalLayout_7.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_4 = QtWidgets.QLabel(self.groupBox)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_4.addWidget(self.label_4)
        self.y2SpinBox = QtWidgets.QSpinBox(self.groupBox)
        self.y2SpinBox.setMaximum(65536)
        self.y2SpinBox.setObjectName("y2SpinBox")
        self.horizontalLayout_4.addWidget(self.y2SpinBox)
        self.horizontalLayout_7.addLayout(self.horizontalLayout_4)
        self.verticalLayout.addLayout(self.horizontalLayout_7)
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.label_5 = QtWidgets.QLabel(self.groupBox)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_9.addWidget(self.label_5)
        self.frameSpinBox = QtWidgets.QSpinBox(self.groupBox)
        self.frameSpinBox.setMinimum(-1)
        self.frameSpinBox.setMaximum(65536)
        self.frameSpinBox.setObjectName("frameSpinBox")
        self.horizontalLayout_9.addWidget(self.frameSpinBox)
        self.horizontalLayout_10.addLayout(self.horizontalLayout_9)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.label_6 = QtWidgets.QLabel(self.groupBox)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_8.addWidget(self.label_6)
        self.pmSpinBox = QtWidgets.QSpinBox(self.groupBox)
        self.pmSpinBox.setSuffix("")
        self.pmSpinBox.setMinimum(0)
        self.pmSpinBox.setMaximum(65536)
        self.pmSpinBox.setObjectName("pmSpinBox")
        self.horizontalLayout_8.addWidget(self.pmSpinBox)
        self.horizontalLayout_10.addLayout(self.horizontalLayout_8)
        self.verticalLayout.addLayout(self.horizontalLayout_10)
        self.horizontalLayout_11.addLayout(self.verticalLayout)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_8 = QtWidgets.QLabel(self.groupBox)
        self.label_8.setObjectName("label_8")
        self.horizontalLayout_5.addWidget(self.label_8)
        self.dirLineEdit = QtWidgets.QLineEdit(self.groupBox)
        self.dirLineEdit.setObjectName("dirLineEdit")
        self.horizontalLayout_5.addWidget(self.dirLineEdit)
        self.dirButton = QtWidgets.QPushButton(self.groupBox)
        self.dirButton.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/folder"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.dirButton.setIcon(icon1)
        self.dirButton.setFlat(True)
        self.dirButton.setObjectName("dirButton")
        self.horizontalLayout_5.addWidget(self.dirButton)
        self.verticalLayout_2.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_13.setObjectName("horizontalLayout_13")
        self.label_7 = QtWidgets.QLabel(self.groupBox)
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_13.addWidget(self.label_7)
        self.reLineEdit = QtWidgets.QLineEdit(self.groupBox)
        self.reLineEdit.setObjectName("reLineEdit")
        self.horizontalLayout_13.addWidget(self.reLineEdit)
        self.verticalLayout_2.addLayout(self.horizontalLayout_13)
        self.onlineCheckBox = QtWidgets.QCheckBox(self.groupBox)
        self.onlineCheckBox.setObjectName("onlineCheckBox")
        self.verticalLayout_2.addWidget(self.onlineCheckBox)
        self.horizontalLayout_11.addLayout(self.verticalLayout_2)
        self.verticalLayout_3.addLayout(self.horizontalLayout_11)
        self.intTable = QtWidgets.QTableWidget(self.groupBox)
        self.intTable.setObjectName("intTable")
        self.intTable.setColumnCount(0)
        self.intTable.setRowCount(0)
        self.verticalLayout_3.addWidget(self.intTable)
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        self.runButton = QtWidgets.QPushButton(self.groupBox)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/run"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.runButton.setIcon(icon2)
        self.runButton.setObjectName("runButton")
        self.horizontalLayout_12.addWidget(self.runButton)
        self.stopButton = QtWidgets.QPushButton(self.groupBox)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/stop"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.stopButton.setIcon(icon3)
        self.stopButton.setObjectName("stopButton")
        self.horizontalLayout_12.addWidget(self.stopButton)
        self.saveButton = QtWidgets.QPushButton(self.groupBox)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/save"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.saveButton.setIcon(icon4)
        self.saveButton.setObjectName("saveButton")
        self.horizontalLayout_12.addWidget(self.saveButton)
        self.copyButton = QtWidgets.QPushButton(self.groupBox)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/copy"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.copyButton.setIcon(icon5)
        self.copyButton.setObjectName("copyButton")
        self.horizontalLayout_12.addWidget(self.copyButton)
        self.exitButton = QtWidgets.QPushButton(self.groupBox)
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/exit"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.exitButton.setIcon(icon6)
        self.exitButton.setObjectName("exitButton")
        self.horizontalLayout_12.addWidget(self.exitButton)
        self.verticalLayout_3.addLayout(self.horizontalLayout_12)
        self.horizontalLayout_14.addWidget(self.groupBox)
        self.plotView = PlotWidget(WSleuth)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.plotView.sizePolicy().hasHeightForWidth())
        self.plotView.setSizePolicy(sizePolicy)
        self.plotView.setObjectName("plotView")
        self.horizontalLayout_14.addWidget(self.plotView)

        self.retranslateUi(WSleuth)
        QtCore.QMetaObject.connectSlotsByName(WSleuth)

    def retranslateUi(self, WSleuth):
        _translate = QtCore.QCoreApplication.translate
        WSleuth.setWindowTitle(_translate("WSleuth", "Sleuth"))
        self.groupBox.setTitle(_translate("WSleuth", "Parameters"))
        self.label.setText(_translate("WSleuth", "x1"))
        self.label_2.setText(_translate("WSleuth", "y1"))
        self.label_3.setText(_translate("WSleuth", "x2"))
        self.label_4.setText(_translate("WSleuth", "y2"))
        self.label_5.setText(_translate("WSleuth", "N"))
        self.label_6.setText(_translate("WSleuth", "<html><head/><body><p>&plusmn;</p></body></html>"))
        self.label_8.setText(_translate("WSleuth", "Folder"))
        self.label_7.setText(_translate("WSleuth", "Regexp"))
        self.onlineCheckBox.setText(_translate("WSleuth", "Online"))
        self.runButton.setText(_translate("WSleuth", "Run"))
        self.stopButton.setText(_translate("WSleuth", "Stop"))
        self.saveButton.setText(_translate("WSleuth", "Save"))
        self.copyButton.setText(_translate("WSleuth", "Copy"))
        self.exitButton.setText(_translate("WSleuth", "Exit"))

from pyqtgraph import PlotWidget
from . import resources_rc



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import sys
import subprocess
from PyQt5 import QtCore, QtGui, QtWidgets
from .ui.ui_snbltb import Ui_WSnblToolBox
from .ui.ui_wsnbltbabout import Ui_WSnblAbout
from .wcrysis import WCrysis
from .convert import ConvertWizard
from .wheader import WHeader
from .wsleuth import WSleuth
from .roerik.wgunnar import WGunnar
from .wdarth import WDarth
# noinspection PyUnresolvedReferences
from .ui import resources_rc

try:
    from . import frozen
except ImportError:
    frozen = False


def get_hg_hash():
    if not get_hg_hash.hash:
        if hasattr(sys, 'frozen') or frozen:
            # noinspection PyUnresolvedReferences
            get_hg_hash.hash = frozen.hg_hash
        else:
            path = os.path.dirname(os.path.dirname(__file__))
            try:
                pipe = subprocess.Popen(['hg', 'id', '-i', '-R', path], stdout=subprocess.PIPE)
                get_hg_hash.hash = pipe.stdout.read().decode()
            except OSError:
                get_hg_hash.hash = 'unknown'
        get_hg_hash.hash = get_hg_hash.hash.strip()
    return get_hg_hash.hash
get_hg_hash.hash = ''


class WAboutSnblTB(QtWidgets.QDialog, Ui_WSnblAbout):
    def __init__(self, parent=None):
        QtWidgets.QDialog.__init__(self, parent)
        self._parent = parent
        self.setupUi(self)
        lt = self.aboutLabel.text()
        lt = lt.replace('@', get_hg_hash())
        self.aboutLabel.setText(lt)
        self.setWindowIcon(QtGui.QIcon(':/swiss'))
        # noinspection PyCallByClass,PyTypeChecker
        QtCore.QTimer.singleShot(0.1, lambda: self.resize(0, 0))

    @QtCore.pyqtSlot()
    def on_closeButton_clicked(self):
        self.close()

    @QtCore.pyqtSlot()
    def on_aboutQtButton_clicked(self):
        # noinspection PyCallByClass,PyTypeChecker,PyArgumentList
        QtWidgets.QMessageBox.aboutQt(self)


class WSnblTB(QtWidgets.QDialog, Ui_WSnblToolBox):
    def __init__(self, parent=None):
        QtWidgets.QDialog.__init__(self, parent)
        self._parent = parent
        self.setupUi(self)
        self.loadSettings()
        self.wcrysis = WCrysis(self)
        self.wconvert = ConvertWizard(self)
        self.wabout = WAboutSnblTB(self)
        self.wheader = WHeader(self)
        self.wsleuth = WSleuth(self)
        self.wroerik = WGunnar(self)
        self.wdarth = WDarth(self)
        # noinspection PyArgumentList
        if QtWidgets.QApplication.screens()[0].logicalDotsPerInch() > 120:  # Hack for HiDPI displays
            size = QtCore.QSize(146, 146)
            self.roerikButton.setIconSize(size)
            self.crysisButton.setIconSize(size)
            self.convertButton.setIconSize(size)
            self.infoButton.setIconSize(size)
            self.headExButton.setIconSize(size)
            self.darthButton.setIconSize(size)
            self.sleuthButton.setIconSize(size)
            self.sigmasButton.setIconSize(size)
        # noinspection PyCallByClass,PyTypeChecker
        QtCore.QTimer.singleShot(0.1, lambda: self.resize(0, 0))

    def keyPressEvent(self, event):
        key = event.key()
        if key == QtCore.Qt.Key_Escape:
            pass
        else:
            QtWidgets.QDialog.keyPressEvent(self, event)

    def closeEvent(self, event):
        self.saveSettings()
        self.wcrysis.close()
        self.wconvert.close()
        self.wabout.close()
        self.wheader.close()
        self.wsleuth.close()
        self.wroerik.close()
        self.wdarth.close()

    def saveSettings(self):
        settings = QtCore.QSettings()
        settings.setValue('WSNBLToolBox/Geometry', self.saveGeometry())

    def loadSettings(self):
        settings = QtCore.QSettings()
        self.restoreGeometry(settings.value('WSNBLToolBox/Geometry', QtCore.QByteArray()))

    @QtCore.pyqtSlot()
    def on_crysisButton_clicked(self):
        self.wcrysis.show()

    @QtCore.pyqtSlot()
    def on_convertButton_clicked(self):
        if self.wconvert.stopped:
            self.wconvert = ConvertWizard(self)
        self.wconvert.show()

    @QtCore.pyqtSlot()
    def on_sigmasButton_clicked(self):
        # noinspection PyArgumentList,PyCallByClass,PyTypeChecker
        QtWidgets.QMessageBox.question(self, 'Fit2D?!', 'Fit2D?! In 2016?! Seriously?! No way!',
                                       QtWidgets.QMessageBox.No)

    @QtCore.pyqtSlot()
    def on_infoButton_clicked(self):
        self.wabout.show()

    @QtCore.pyqtSlot()
    def on_headExButton_clicked(self):
        self.wheader.show()

    @QtCore.pyqtSlot()
    def on_sleuthButton_clicked(self):
        self.wsleuth.show()

    @QtCore.pyqtSlot()
    def on_darthButton_clicked(self):
        self.wdarth.show()

    @QtCore.pyqtSlot()
    def on_roerikButton_clicked(self):
        self.wroerik.show()



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import glob
import collections
from datetime import datetime
from PyQt5 import QtCore, QtWidgets
from cryio import cbfimage, crysalis
from .ui.ui_wcrysis import Ui_WCrysis
from . import pyqt2bool


PROGRESS_STEP = 60


Scan = collections.namedtuple('Scan', ['kappa', 'omegaphi', 'type'])


class WorkerSignals(QtCore.QObject):
    errorSignal = QtCore.pyqtSignal(str)
    progressSignal = QtCore.pyqtSignal(int)
    stopSignal = QtCore.pyqtSignal()
    finishedSignal = QtCore.pyqtSignal(object)


class Worker(QtCore.QRunnable):
    corruptKeys = 'Omega', 'Phi', 'Kappa', 'Oscillation_axis', 'Phi_increment', 'Omega_increment'

    def __init__(self, folder, deleteAngles):
        QtCore.QRunnable.__init__(self)
        self.stop = False
        self.folder = folder
        self.deleteAngles = deleteAngles
        self.signals = WorkerSignals()
        self.signals.stopSignal.connect(self.stopIt)

    def stopIt(self):
        self.stop = True

    def run(self):
        self.basename = os.path.basename(self.folder)
        self.runname = os.path.join(self.folder, self.basename)
        self.cbflist = sorted(glob.glob(os.path.join(self.folder, '*.cbf')))
        self.ncbf = len(self.cbflist)
        self.scans = collections.OrderedDict()
        self.readFiles()
        self.createRunFiles()
        self.stopIt()
        self.signals.finishedSignal.emit(self)

    def readFiles(self):
        header, i = None, 0
        for i, cbf in enumerate(self.cbflist, 1):
            # noinspection PyArgumentList
            QtCore.QCoreApplication.processEvents()
            if self.stop:
                return
            try:
                header = cbfimage.CbfHeader(cbf)
            except cbfimage.CbfError:
                self.signals.errorSignal.emit('File {} is corrupted. Cannot proceed further'.format(cbf))
                self.stopIt()
                return
            if 'Oscillation_axis' not in header.header_dict:
                if 'Start_angle' in header.header_dict and 'Angle_increment' in header.header_dict:
                    header.header_dict['Oscillation_axis'] = 'OMEGA'
                    header.header_dict['Kappa'] = 0
                    header.header_dict['Phi'] = 0
                else:
                    self.signals.errorSignal.emit(self.folder)
                    self.stopIt()
                    return
            header.header_dict['cbf'] = os.path.basename(cbf)
            axis = header.header_dict['Oscillation_axis'].upper()
            scan = Scan(header['Kappa'], header['Phi' if axis == 'OMEGA' else 'Omega'], axis)
            if scan not in self.scans:
                self.scans[scan] = []
            self.scans[scan].append(header.header_dict)
            if i == 1:
                self.wavelength = header['Wavelength']
                self.dist = header['Detector_distance'] * 1000
                self.center_x, self.center_y = header['Beam_xy']
            if self.deleteAngles:
                self.removeCorruptKeys(header, cbf)
            if not i % PROGRESS_STEP:
                self.signals.progressSignal.emit(PROGRESS_STEP)
        if header:
            try:
                self.shape = (header.binary_header_dict['X-Binary-Size-Second-Dimension'],
                              header.binary_header_dict['X-Binary-Size-Fastest-Dimension'])
            except KeyError:
                self.shape = None
        if i % PROGRESS_STEP:
            self.signals.progressSignal.emit(i % PROGRESS_STEP)

    def removeCorruptKeys(self, header, name):
        for key in self.corruptKeys:
            if key in header.header_dict:
                del header.header_dict[key]
        header.save_cbf(name)

    # noinspection PyTypeChecker
    def createRunFiles(self):
        if self.stop or not len(self.scans):
            return
        runFile = []
        for i, scan in enumerate(self.scans):
            scanset = self.scans[scan]
            scanset.sort(key=lambda k: k['Start_angle'])
            dscr = crysalis.RunDscr(i)
            dscr.axis = crysalis.SCAN_AXIS[scan.type]
            dscr.kappa = scan.kappa
            dscr.omegaphi = scan.omegaphi
            dscr.start = scanset[0]['Start_angle']
            dscr.end = scanset[-1]['Start_angle']
            dscr.width = scanset[0]['Angle_increment']
            dscr.todo = dscr.done = len(scanset)
            dscr.exposure = scanset[0]['Exposure_time']
            runFile.append(dscr)
        runHeader = crysalis.RunHeader(self.basename.encode(), self.folder.encode(), len(self.scans))
        crysalis.saveRun(self.runname, runHeader, runFile)
        if self.shape is None:
            crysalis.saveSet(self.runname)
            crysalis.saveCcd(self.runname)
            crysalis.saveSel(self.runname)
        elif self.shape == cbfimage.PILATUS2M:
            crysalis.saveSet(self.runname, '2M')
            crysalis.saveCcd(self.runname)
            crysalis.saveSel(self.runname)
        elif self.shape == cbfimage.PILATUS6M:
            crysalis.saveSet(self.runname, '6M')
            crysalis.saveCcd(self.runname, '6M')
            crysalis.saveSel(self.runname, 'esperanto')
        elif self.shape == cbfimage.PILATUS300K:
            crysalis.saveSet(self.runname, '300K')
            crysalis.saveCcd(self.runname)
            crysalis.saveSel(self.runname)
        elif self.shape == cbfimage.PILATUS1M:
            crysalis.saveSet(self.runname, '1M')
            crysalis.saveCcd(self.runname, '1M')
            crysalis.saveSel(self.runname, 'esperanto')
        crysalis.savePar(self.runname, {'wavelength': self.wavelength, 'path': self.folder, 'center_y': self.center_y,
                                        'center_x': self.center_x, 'basename': self.basename, 'dist': self.dist,
                                        'date': datetime.now().strftime(u'%Y-%m-%d %H:%M:%S'),
                                        'chip': True, 'chip1': self.shape[0], 'chip2': self.shape[1]},)
        crysalis.saveAliases(self.folder, {'scans': self.scans, 'basename': self.basename, 'crysis': True})
        crysalis.saveCrysalisExpSettings(self.folder)


class CrysisWorker(QtCore.QObject):
    progressSignal = QtCore.pyqtSignal(int)
    finishedSignal = QtCore.pyqtSignal(list)

    def __init__(self, parent=None):
        QtCore.QObject.__init__(self, parent)
        self.pool = QtCore.QThreadPool.globalInstance()
        self.workers, self.errors = [], []

    def run(self, folder, deleteAngles):
        self.files = self.done = 0
        self.workers, self.errors = [], []
        for dirpath, dirnames, filenames in os.walk(folder):
            self.files += len(filenames)
            worker = Worker(dirpath, deleteAngles)
            worker.signals.errorSignal.connect(self.workerError)
            worker.signals.progressSignal.connect(self.workerProgress)
            worker.signals.finishedSignal.connect(self.workerFinished)
            self.workers.append(worker)
        self.progressSignal.emit(0)
        for worker in self.workers:
            self.pool.start(worker)

    def stopIt(self):
        for worker in self.workers:
            worker.signals.stopSignal.emit()

    def workerProgress(self, i):
        self.done += i
        done = 100.0 * self.done / self.files
        self.progressSignal.emit(done)

    def workerError(self, folder):
        self.errors.append(folder)

    def workerFinished(self, worker):
        self.workers.remove(worker)
        if not self.workers:
            self.progressSignal.emit(100)
            self.finishedSignal.emit(self.errors)


class WCrysis(QtWidgets.QDialog, Ui_WCrysis):
    stopWorkerSignal = QtCore.pyqtSignal()
    runWorkerSignal = QtCore.pyqtSignal(str, bool)

    def __init__(self, parent=None):
        QtWidgets.QDialog.__init__(self, parent)
        self._parent = parent
        self.setupUi(self)
        self.loadSettings()
        self.stopButton.setVisible(False)
        self.workerThread = QtCore.QThread()
        self.worker = CrysisWorker()
        self.worker.moveToThread(self.workerThread)
        self.worker.progressSignal.connect(self.showProgress)
        self.worker.finishedSignal.connect(self.workerFinished)
        self.stopWorkerSignal.connect(self.worker.stopIt)
        self.runWorkerSignal.connect(self.worker.run)
        self.workerThread.start()

    def showError(self, errors):
        # noinspection PyCallByClass,PyArgumentList,PyTypeChecker
        QtWidgets.QMessageBox.critical(self,
                                       'Crysis error',
                                       'Datasets in folders\n{}\nare corrupted: it seems you have aborted the '
                                       'experiments because the headers are empty.\n It cannot be '
                                       'processed.'.format('\n'.join(errors)))

    def showProgress(self, value):
        self.runProgressBar.setFormat('{0}: %p%'.format('Reading files'))
        self.runProgressBar.setValue(value)

    def closeEvent(self, event):
        self.saveSettings()
        self.stopWorkerSignal.emit()
        self.hide()

    def keyPressEvent(self, event):
        key = event.key()
        if key == QtCore.Qt.Key_Escape:
            pass
        else:
            QtWidgets.QDialog.keyPressEvent(self, event)

    def saveSettings(self):
        settings = QtCore.QSettings()
        settings.setValue('WCrysis/Geometry', self.saveGeometry())
        settings.setValue('WCrysis/lastFolder', self.lastFolder)
        settings.setValue('WCrysis/folder', self.folderLineEdit.text())
        settings.setValue('WCrysis/delAngles', self.delAnglesCheckBox.isChecked())

    def loadSettings(self):
        settings = QtCore.QSettings()
        self.restoreGeometry(settings.value('WCrysis/Geometry', QtCore.QByteArray()))
        self.lastFolder = settings.value('WCrysis/lastFolder', u'')
        self.folderLineEdit.setText(settings.value('WCrysis/folder', u''))
        self.delAnglesCheckBox.setChecked(pyqt2bool(settings.value('WCrysis/delAngles', True)))

    @QtCore.pyqtSlot()
    def on_runButton_clicked(self):
        folder = self.folderLineEdit.text()
        if not folder or not os.path.exists(folder):
            return
        self.runButton.setVisible(False)
        self.stopButton.setVisible(True)
        self.runWorkerSignal.emit(folder, self.delAnglesCheckBox.isChecked())

    def workerFinished(self, errors):
        self.runButton.setVisible(True)
        self.stopButton.setVisible(False)
        if errors:
            self.showError(errors)

    @QtCore.pyqtSlot()
    def on_stopButton_clicked(self):
        self.stopWorkerSignal.emit()

    @QtCore.pyqtSlot()
    def on_folderButton_clicked(self):
        # noinspection PyCallByClass,PyTypeChecker
        folder = QtWidgets.QFileDialog.getExistingDirectory(self, 'Select directory', self.lastFolder)
        if not folder:
            return
        self.folderLineEdit.setText(folder)
        self.lastFolder = folder



# -*- coding: utf-8 -*-


from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_WHeader(object):
    def setupUi(self, WHeader):
        WHeader.setObjectName("WHeader")
        WHeader.resize(521, 421)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/header"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        WHeader.setWindowIcon(icon)
        WHeader.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.verticalLayout = QtWidgets.QVBoxLayout(WHeader)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(WHeader)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.folderLineEdit = QtWidgets.QLineEdit(WHeader)
        self.folderLineEdit.setObjectName("folderLineEdit")
        self.horizontalLayout.addWidget(self.folderLineEdit)
        self.folderButton = QtWidgets.QToolButton(WHeader)
        self.folderButton.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/folder"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.folderButton.setIcon(icon1)
        self.folderButton.setObjectName("folderButton")
        self.horizontalLayout.addWidget(self.folderButton)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.valuesList = QtWidgets.QListWidget(WHeader)
        self.valuesList.setObjectName("valuesList")
        self.verticalLayout.addWidget(self.valuesList)
        self.resultTable = QtWidgets.QTableWidget(WHeader)
        self.resultTable.setObjectName("resultTable")
        self.resultTable.setColumnCount(0)
        self.resultTable.setRowCount(0)
        self.verticalLayout.addWidget(self.resultTable)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.runButton = QtWidgets.QPushButton(WHeader)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/run"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.runButton.setIcon(icon2)
        self.runButton.setObjectName("runButton")
        self.horizontalLayout_2.addWidget(self.runButton)
        self.stopButton = QtWidgets.QPushButton(WHeader)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/stop"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.stopButton.setIcon(icon3)
        self.stopButton.setObjectName("stopButton")
        self.horizontalLayout_2.addWidget(self.stopButton)
        self.saveButton = QtWidgets.QPushButton(WHeader)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/save"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.saveButton.setIcon(icon4)
        self.saveButton.setObjectName("saveButton")
        self.horizontalLayout_2.addWidget(self.saveButton)
        self.runProgressBar = QtWidgets.QProgressBar(WHeader)
        self.runProgressBar.setProperty("value", 0)
        self.runProgressBar.setObjectName("runProgressBar")
        self.horizontalLayout_2.addWidget(self.runProgressBar)
        self.verticalLayout.addLayout(self.horizontalLayout_2)

        self.retranslateUi(WHeader)
        QtCore.QMetaObject.connectSlotsByName(WHeader)

    def retranslateUi(self, WHeader):
        _translate = QtCore.QCoreApplication.translate
        WHeader.setWindowTitle(_translate("WHeader", "Header Extractor"))
        self.label.setText(_translate("WHeader", "Folder"))
        self.runButton.setText(_translate("WHeader", "Run"))
        self.stopButton.setText(_translate("WHeader", "Stop"))
        self.saveButton.setText(_translate("WHeader", "Save text"))

from . import resources_rc



# -*- coding: utf-8 -*-


from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_WSnblAbout(object):
    def setupUi(self, WSnblAbout):
        WSnblAbout.setObjectName("WSnblAbout")
        WSnblAbout.resize(863, 485)
        self.verticalLayout = QtWidgets.QVBoxLayout(WSnblAbout)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.label = QtWidgets.QLabel(WSnblAbout)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setMinimumSize(QtCore.QSize(191, 71))
        self.label.setMaximumSize(QtCore.QSize(191, 71))
        self.label.setText("")
        self.label.setPixmap(QtGui.QPixmap(":/snbl"))
        self.label.setScaledContents(True)
        self.label.setObjectName("label")
        self.horizontalLayout_2.addWidget(self.label)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem2)
        self.aboutLabel = QtWidgets.QLabel(WSnblAbout)
        self.aboutLabel.setObjectName("aboutLabel")
        self.horizontalLayout_3.addWidget(self.aboutLabel)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem3)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem4)
        self.aboutQtButton = QtWidgets.QPushButton(WSnblAbout)
        self.aboutQtButton.setObjectName("aboutQtButton")
        self.horizontalLayout.addWidget(self.aboutQtButton)
        self.closeButton = QtWidgets.QPushButton(WSnblAbout)
        self.closeButton.setObjectName("closeButton")
        self.horizontalLayout.addWidget(self.closeButton)
        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(WSnblAbout)
        QtCore.QMetaObject.connectSlotsByName(WSnblAbout)

    def retranslateUi(self, WSnblAbout):
        _translate = QtCore.QCoreApplication.translate
        WSnblAbout.setWindowTitle(_translate("WSnblAbout", "About SNBL Tool Box"))
        self.aboutLabel.setText(_translate("WSnblAbout", "<html><head/><body><p>SNBL Tool Box v1.0 (Swiss knife for Pilatus 2M data)</p><p>(c) Vadim Dyadkin</p><p>This program is licensed under GPL v3</p><p>Mercurial repository: <a href=\"http://hg.3lp.cx/snbltb\"><span style=\" text-decoration: underline; color:#0057ae;\">http://hg.3lp.cx/snbltb</span></a></p><p>Mercurial hash: @</p><p>When you use this software, please quote the following reference:</p><p><a href=\"http://dx.doi.org/10.1107/S1600577516002411\"><span style=\" text-decoration: underline; color:#0057ae;\">http://dx.doi.org/10.1107/S1600577516002411</a></p></body></html>"))
        self.aboutQtButton.setText(_translate("WSnblAbout", "About Qt"))
        self.closeButton.setText(_translate("WSnblAbout", "Close"))

from . import resources_rc



#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
from PyQt5 import QtWidgets


def pyqt2bool(entry):
    return not (entry == 'false' or not entry)


def main():
    app = QtWidgets.QApplication(sys.argv)
    app.setOrganizationName('SNBL')
    app.setOrganizationDomain('snbl.eu')
    app.setApplicationName('pylatus')
    from SNBLToolBox.wsnbltb import WSnblTB
    wsnbl = WSnblTB()
    wsnbl.show()
    sys.exit(app.exec_())


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-


from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_WDarth(object):
    def setupUi(self, WDarth):
        WDarth.setObjectName("WDarth")
        WDarth.resize(652, 513)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/darth"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        WDarth.setWindowIcon(icon)
        WDarth.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(WDarth)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.groupBox = QtWidgets.QGroupBox(WDarth)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(self.groupBox)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.folderLineEdit = QtWidgets.QLineEdit(self.groupBox)
        self.folderLineEdit.setObjectName("folderLineEdit")
        self.horizontalLayout.addWidget(self.folderLineEdit)
        self.folderButton = QtWidgets.QPushButton(self.groupBox)
        self.folderButton.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/folder"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.folderButton.setIcon(icon1)
        self.folderButton.setObjectName("folderButton")
        self.horizontalLayout.addWidget(self.folderButton)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.volumeCheckBox = QtWidgets.QCheckBox(self.groupBox)
        self.volumeCheckBox.setChecked(True)
        self.volumeCheckBox.setObjectName("volumeCheckBox")
        self.horizontalLayout_2.addWidget(self.volumeCheckBox)
        self.label_3 = QtWidgets.QLabel(self.groupBox)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_2.addWidget(self.label_3)
        self.xLineEdit = QtWidgets.QLineEdit(self.groupBox)
        self.xLineEdit.setMaxLength(4)
        self.xLineEdit.setObjectName("xLineEdit")
        self.horizontalLayout_2.addWidget(self.xLineEdit)
        self.label_4 = QtWidgets.QLabel(self.groupBox)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_2.addWidget(self.label_4)
        self.yLineEdit = QtWidgets.QLineEdit(self.groupBox)
        self.yLineEdit.setMaxLength(4)
        self.yLineEdit.setObjectName("yLineEdit")
        self.horizontalLayout_2.addWidget(self.yLineEdit)
        self.label_5 = QtWidgets.QLabel(self.groupBox)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_2.addWidget(self.label_5)
        self.zLineEdit = QtWidgets.QLineEdit(self.groupBox)
        self.zLineEdit.setMaxLength(4)
        self.zLineEdit.setObjectName("zLineEdit")
        self.horizontalLayout_2.addWidget(self.zLineEdit)
        self.label_6 = QtWidgets.QLabel(self.groupBox)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_2.addWidget(self.label_6)
        self.dQLineEdit = QtWidgets.QLineEdit(self.groupBox)
        self.dQLineEdit.setMaxLength(4)
        self.dQLineEdit.setObjectName("dQLineEdit")
        self.horizontalLayout_2.addWidget(self.dQLineEdit)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.treeLayers = MTreeWidget(self.groupBox)
        self.treeLayers.setObjectName("treeLayers")
        self.verticalLayout.addWidget(self.treeLayers)
        self.lorentzCheckBox = QtWidgets.QCheckBox(self.groupBox)
        self.lorentzCheckBox.setObjectName("lorentzCheckBox")
        self.verticalLayout.addWidget(self.lorentzCheckBox)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_2 = QtWidgets.QLabel(self.groupBox)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_4.addWidget(self.label_2)
        self.editDownsample = QtWidgets.QLineEdit(self.groupBox)
        self.editDownsample.setObjectName("editDownsample")
        self.horizontalLayout_4.addWidget(self.editDownsample)
        self.label_7 = QtWidgets.QLabel(self.groupBox)
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_4.addWidget(self.label_7)
        self.editScale = QtWidgets.QLineEdit(self.groupBox)
        self.editScale.setObjectName("editScale")
        self.horizontalLayout_4.addWidget(self.editScale)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.runButton = QtWidgets.QPushButton(self.groupBox)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/run"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.runButton.setIcon(icon2)
        self.runButton.setObjectName("runButton")
        self.horizontalLayout_3.addWidget(self.runButton)
        self.stopButton = QtWidgets.QPushButton(self.groupBox)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/stop"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.stopButton.setIcon(icon3)
        self.stopButton.setObjectName("stopButton")
        self.horizontalLayout_3.addWidget(self.stopButton)
        self.runProgressBar = QtWidgets.QProgressBar(self.groupBox)
        self.runProgressBar.setProperty("value", 0)
        self.runProgressBar.setObjectName("runProgressBar")
        self.horizontalLayout_3.addWidget(self.runProgressBar)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.verticalLayout_3.addWidget(self.groupBox)

        self.retranslateUi(WDarth)
        QtCore.QMetaObject.connectSlotsByName(WDarth)

    def retranslateUi(self, WDarth):
        _translate = QtCore.QCoreApplication.translate
        WDarth.setWindowTitle(_translate("WDarth", "Darth Vader\'s Chimera"))
        self.groupBox.setTitle(_translate("WDarth", "Create ccp4 map"))
        self.label.setText(_translate("WDarth", "Folder with processed by Crysalis cbf files:"))
        self.folderLineEdit.setToolTip(_translate("WDarth", "<html><head/><body><p>Choose here the folder where there are your data sets</p></body></html>"))
        self.volumeCheckBox.setText(_translate("WDarth", "Reconstruct volume:"))
        self.label_3.setText(_translate("WDarth", "x"))
        self.xLineEdit.setText(_translate("WDarth", "256"))
        self.label_4.setText(_translate("WDarth", "y"))
        self.yLineEdit.setText(_translate("WDarth", "256"))
        self.label_5.setText(_translate("WDarth", "z"))
        self.zLineEdit.setText(_translate("WDarth", "256"))
        self.label_6.setText(_translate("WDarth", "dQ"))
        self.dQLineEdit.setText(_translate("WDarth", "0.6"))
        self.treeLayers.headerItem().setText(0, _translate("WDarth", "hkl_1"))
        self.treeLayers.headerItem().setText(1, _translate("WDarth", "hkl_2"))
        self.treeLayers.headerItem().setText(2, _translate("WDarth", "hkl_c"))
        self.treeLayers.headerItem().setText(3, _translate("WDarth", "Qmax"))
        self.treeLayers.headerItem().setText(4, _translate("WDarth", "thickness"))
        self.lorentzCheckBox.setText(_translate("WDarth", "Apply Lorentz corrections"))
        self.label_2.setText(_translate("WDarth", "Downsample"))
        self.label_7.setText(_translate("WDarth", "Scale"))
        self.runButton.setText(_translate("WDarth", "Run"))
        self.stopButton.setText(_translate("WDarth", "Stop"))

from ..mtreewidget import MTreeWidget
from . import resources_rc



#!/usr/bin/python
# -*- coding: utf-8 -*-

import math
import numpy as np
from scipy.linalg import cholesky


__all__ = ['neighbors']


PILATUS_X, PILATUS_Y = 1475, 1679
PIXEL = np.linalg.inv(cholesky(np.array((
    (1., 0., 0.),
    (0., 172e-3 ** 2, 0.),
    (0., 0., 172e-3 ** 2)
))))


def rx(a):
    """Rotational matrix along x"""
    return np.array((
        (1, 0, 0),
        (0, math.cos(a), math.sin(a)),
        (0, -math.sin(a), math.cos(a)),
    ))


def ry(a):
    """Rotational matrix along y"""
    return np.array((
        (math.cos(a), 0, -math.sin(a)),
        (0, 1, 0),
        (math.sin(a), 0, math.cos(a)),
    ))


def rz(a):
    """Rotational matrix along z"""
    return np.array((
        (math.cos(a), math.sin(a), 0),
        (-math.sin(a), math.cos(a), 0),
        (0, 0, 1),
    ))


def rotation1(alpha, beta, omega, kappa, phi):
    t1 = np.dot(np.dot(ry(beta), rz(-phi)), ry(-beta))
    t2 = np.dot(np.dot(ry(alpha), rz(-kappa)), ry(-alpha))
    return np.dot(np.dot(t1, t2), rz(-omega))


def rotation2(alpha, beta, omega, kappa, phi):
    s1 = np.dot(np.dot(ry(beta), rz(phi)), ry(-beta))
    s2 = np.dot(np.dot(ry(alpha), rz(kappa)), ry(-alpha))
    return np.dot(np.dot(rz(omega), s2), s1)


def ifix(v):
    return int(math.ceil(v) if v < 0 else math.floor(v))


def t1(x, y, z):
    h1 = (z - x) / y
    h2 = -(z + x) / y
    return ifix(min(h1, h2)), ifix(max(h1, h2)) + 1


def t2(x, y):
    if x >= 0 and y >= 0:
        return 1
    elif x < 0 and y < 0:
        return 2
    else:
        return 3


def t3(x, y, i):
    ip, im = ifix(x), ifix(y)
    _id = abs(ip - im)
    if _id:
        k0 = 1
        k1 = min(ip, im) + (1 if i == 1 else 0)
        k2 = k1 + _id - (0 if i == 3 else 1)
    else:
        k0 = 1 if i == 3 else 0
        k1 = k2 = 0
    return k1, k2 + 1, k0


def tanaka1(b, vecb, radius, delta):
    alpha = b[2, 2] * vecb[2]
    beta = b[2, 2]
    rp = radius + delta
    return t1(alpha, beta, rp)


def tanaka2(b, vecb, radius, delta, l):
    rp, rm = [i ** 2 for i in (radius + delta, radius - delta)]
    r1 = (b[2, 2] * vecb[2] + b[2, 2] * l) ** 2
    r3p = 0 if r1 >= rp else math.sqrt(rp - r1)
    if r1 < rm:
        r3m = math.sqrt(rm - r1)
        ir3 = 1
    else:
        r3m = ir3 = 0
    alpha = b[1, 1] * vecb[1] + b[1, 2] * (vecb[2] + l)
    beta = b[1, 1]
    return (r3p, r3m, ir3), t1(alpha, beta, r3p)


def tanaka3(b, vecb, r3, k, l):
    hklgen = []
    r3p, r3m, ir3 = [i ** 2 for i in r3]
    r1 = (b[1, 1] * vecb[1] + b[1, 2] * (vecb[2] + l) + b[1, 1] * k) ** 2
    r2p = 0 if r1 >= r3p else math.sqrt(r3p - r1)
    if r1 >= r3m:
        r2m = ir2 = 0
    else:
        ir2 = 1
        r2m = math.sqrt(r3m - r1)
    beta = b[0, 0]
    alpha = beta * vecb[0] + b[0, 1] * (vecb[1] + k) + b[0, 2] * (vecb[2] + l)
    if ir2:
        h11p = (r2p - alpha) / beta
        h11m = (r2m - alpha) / beta
        h12p = -(r2p + alpha) / beta
        h12m = -(r2m + alpha) / beta
        l1 = t2(h11p, h11m)
        l2 = t2(h12p, h12m)
        i = t3(h11p, h11m, l1)
        j = t3(h12p, h12m, l2)
        if i[2]:
            for h in range(i[0], i[1]):
                hklgen.append((h, k, l))
        if j[2]:
            for h in range(j[0], j[1]):
                hklgen.append((h, k, l))
    else:
        h11p = (r2p - alpha) / beta
        h12p = -(r2p + alpha) / beta
        l1 = t2(h11p, h12p)
        i = t3(h11p, h12p, l1)
        if i[2]:
            for h in range(i[0], i[1]):
                hklgen.append((h, k, l))
    return hklgen


def reflections(b, vecb, radius, halfdelta):
    lmin, lmax = tanaka1(b, vecb, radius, halfdelta)
    for l in range(lmin, lmax):
        r3, klimit = tanaka2(b, vecb, radius, halfdelta, l)
        for k in range(klimit[0], klimit[1]):
            hkl = tanaka3(b, vecb, r3, k, l)
            if hkl:
                for i in hkl:
                    if i != (0, 0, 0):
                        yield i


def neighbors(u, wavelength, delta, alpha, beta, omega, kappa, phi, phix, phiy, x0, y0, d0):
    if not wavelength:
        return None
    alpha, beta, omega, kappa, phi, phix, phiy = [math.radians(i) for i in (alpha, beta, omega, kappa, phi, phix, phiy)]
    halfdelta = delta / 2.
    radius = 1. / wavelength
    awl = np.array((-radius, 0., 0.))
    ubm = u / wavelength
    t = rotation1(alpha, beta, omega, kappa, phi)
    s = rotation2(alpha, beta, omega, kappa, phi)
    vecb = np.dot(np.dot(np.linalg.inv(ubm), t), awl)
    u = np.dot(ry(phiy), rx(phix)).transpose()
    b = cholesky(np.dot(ubm.transpose(), ubm))
    pixelu = np.dot(PIXEL, u)
    subm = np.dot(s, ubm)
    result = []
    for reflection in reflections(b, vecb, radius, halfdelta):
        xyz = np.dot(subm, reflection)
        ubmh = np.dot(ubm, reflection)
        x = 2 * math.asin(xyz[0] / math.sqrt(np.dot(ubmh, ubmh)))
        y = math.atan2(-xyz[2], xyz[1])
        sh = np.array((-math.cos(x), math.sin(x) * math.cos(y), -math.sin(x) * math.sin(y)))
        p = np.dot(pixelu, sh)
        if p[0] < 0:
            p *= -d0 / p[0]
            x, y = [int(math.floor(i)) for i in (p[2] + x0, p[1] + y0)]
            if 0 <= x <= PILATUS_X and 0 <= y <= PILATUS_Y:
                ext = np.linalg.norm(awl + xyz) - radius
                result.append((reflection, (y, x), ext))
    return result


def test():
    wavelength = 0.68894
    U = np.array((
        (0.022579481, -0.066943125, -0.044049163),
        (-0.010769909, 0.11496317, -0.029219808),
        (0.16529107, 0.10117289, 0.0041047721))
    )
    alpha = 50.0
    beta = 0.0
    phix = 0.01914
    phiy = 0.02620
    d0 = 145.94578
    y0, x0 = 537.45944, 744.55962
    omega, kappa, phi = 0, 0., 19.6990
    delta = 0.003
    return neighbors(U, wavelength, delta, alpha, beta, omega, kappa, phi, phix, phiy, x0, y0, d0)


if __name__ == '__main__':
    n = test()
    # import pprint
    # pprint.pprint(n)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from PyQt5 import QtCore, QtWidgets


class MTreeWidget(QtWidgets.QTreeWidget):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.addedLayers = 0

    def mouseDoubleClickEvent(self, event):
        if self.itemAt(event.pos()) is None:
            item = QtWidgets.QTreeWidgetItem(self)
            item.setFlags(item.flags() | QtCore.Qt.ItemIsEditable)
            if self.addedLayers == 0 or self.addedLayers >= 3:
                item.setText(0, '1 0 0')
                item.setText(1, '0 1 0')
            elif self.addedLayers == 1:
                item.setText(0, '1 0 0')
                item.setText(1, '0 0 1')
            elif self.addedLayers == 2:
                item.setText(0, '0 1 0')
                item.setText(1, '0 0 1')
            item.setText(2, '0 0 0')
            item.setText(3, '1.0')
            item.setText(4, '0.01')
            self.addTopLevelItem(item)
            self.addedLayers += 1
        super().mouseDoubleClickEvent(event)

    def mousePressEvent(self, event):
        item = self.itemAt(event.pos())
        if event.button() == QtCore.Qt.RightButton and item is not None:
            self.takeTopLevelItem(self.indexOfTopLevelItem(item))
            self.addedLayers -= 1
        else:
            super().mousePressEvent(event)

    def getLayers(self):
        res = []
        for i in range(self.topLevelItemCount()):
            item = self.topLevelItem(i)
            layer = {
                'hkl1': item.text(0),
                'hkl2': item.text(1),
                'center': item.text(2),
                'qmax': item.text(3),
                'thickness': item.text(4),
            }
            res.append(layer)
        return res



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import re
from PyQt5 import QtCore, QtGui, QtWidgets
from cryio.cbfimage import CbfImage
from . import pyqt2bool
from .ui.ui_wsleuth import Ui_WSleuth


class SleuthWalker(QtCore.QObject):
    progressSignal = QtCore.pyqtSignal(object, float, int, float)
    finishedSignal = QtCore.pyqtSignal()

    def __init__(self, dire, frame, frame_pm, roi, reg, psi):
        QtCore.QObject.__init__(self)
        self.dir = dire
        self.roi = roi
        self.frame = frame
        self.frame_pm = frame_pm
        self.reg = reg
        self.psi = psi
        self.stop()

    def stop(self):
        self.stopped = True

    def run(self):
        self.stopped = False
        if self.frame == -1:
            self.listDir()
        elif self.psi:
            self.psiScan()
        else:
            self.walk()
        self.finishedSignal.emit()
        self.stop()

    def psiScan(self):
        files = {}
        for fcbf in sorted(os.listdir(self.dir)):
            if not fcbf.endswith('.cbf'):
                continue
            QtCore.QCoreApplication.processEvents()
            if self.stopped:
                return
            nums = re.split(r'^psi_([-+]?[0-9]*\.?[0-9]+)_\d+p_(\d+)\.cbf$', fcbf)
            frame = int(nums[2])
            psi = float(nums[1])
            if self.frame - self.frame_pm <= frame <= self.frame + self.frame_pm:
                if psi not in files:
                    files[psi] = []
                files[psi].append(fcbf)
        for psi in sorted(files):
            self.readIntensity(self.dir, files[psi], psi)

    def walk(self):
        params = []
        for dirpath, dirnames, filenames in sorted(os.walk(self.dir)):
            QtCore.QCoreApplication.processEvents()
            if self.stopped:
                return
            if not filenames:
                continue
            match = re.search(self.reg, os.path.basename(dirpath))
            if match:
                try:
                    params.append((dirpath, self.listDirForFrames(filenames), match.group(1)))
                except (ValueError, IndexError):
                    continue

        def sp(key):
            d, f, m = key
            try:
                return float(d)
            except ValueError:
                return d
        params.sort(key=sp)
        for p in params:
            self.readIntensity(*p)

    def listDir(self):
        i = 0
        for fcbf in sorted(os.listdir(self.dir)):
            if not fcbf.endswith('.cbf'):
                continue
            # noinspection PyArgumentList
            QtCore.QCoreApplication.processEvents()
            if self.stopped:
                return
            try:
                frame = int(re.split(r'.+_\d+p_(\d+)\.cbf$', fcbf)[1])
            except IndexError:
                frame_re = re.split(r'.+_(\d+)p\.cbf$', fcbf)
                if len(frame_re) > 1:
                    frame = int(frame_re[1])
                else:
                    frame = i
                    i += 1
            self.readIntensity(self.dir, [fcbf], frame)

    def readIntensity(self, dire, fcbfs, param):
        roi_sum = flux = 0
        for fcbf in fcbfs:
            if not fcbf.endswith('.cbf'):
                continue
            # noinspection PyArgumentList
            QtCore.QCoreApplication.processEvents()
            if self.stopped:
                return
            x1, x2, y1, y2 = self.roi
            cbf = CbfImage(os.path.join(dire, fcbf))
            if 'Flux' not in cbf.header_dict:
                return
            roi = cbf.array[y1:y2, x1:x2]
            roi_sum += roi.sum()
            flux += cbf.header_dict['Flux']
        roi_sum = float(roi_sum)
        try:
            roi_norm = roi_sum / flux
        except ZeroDivisionError:
            roi_norm = 0
        self.progressSignal.emit(param, roi_sum, flux, roi_norm)

    def listDirForFrames(self, filenames):
        start = self.frame - self.frame_pm
        stop = self.frame + self.frame_pm + 1
        # format: base_XXXXXXp_frames.cbf
        rexp = r'.+_\d+p_(%s)\.cbf$' % '|'.join('%05d' % n for n in range(start, stop))
        for fname in sorted(filenames):
            if re.search(rexp, fname):
                yield fname


class WSleuth(QtWidgets.QDialog, Ui_WSleuth):
    stopWalkSignal = QtCore.pyqtSignal()

    def __init__(self, parent=None):
        QtWidgets.QDialog.__init__(self, parent)
        self._parent = parent
        self.setupUi(self)
        self.loadSettings()
        self.stopButton.hide()
        self.walkThread = QtCore.QThread()
        self.createContextMenu()
        self.intTable.keyPressEvent = self.resultTableKeyPressEvent
        self.intTable.contextMenuEvent = self.resultTableContextMenuEvent
        self.onlineCheckBox.setText('Psi scan')

    def createContextMenu(self):
        copyMenuAction = QtWidgets.QAction('&Copy', self.intTable)
        copyMenuAction.setShortcut(QtGui.QKeySequence.Copy)
        # noinspection PyUnresolvedReferences
        copyMenuAction.triggered.connect(self.copyToClipboard)
        saveTextAction = QtWidgets.QAction('&Save as text', self.intTable)
        saveTextAction.setShortcut(QtGui.QKeySequence.Save)
        # noinspection PyUnresolvedReferences
        saveTextAction.triggered.connect(self.on_saveButton_clicked)
        self.contextMenu = QtWidgets.QMenu()
        self.contextMenu.addAction(copyMenuAction)
        self.contextMenu.addAction(saveTextAction)

    def resultTableContextMenuEvent(self, event):
        self.contextMenu.exec_(event.globalPos())

    @QtCore.pyqtSlot()
    def copyToClipboard(self):
        selectedItems = self.intTable.selectedItems()
        toClipboard = u''
        for i in range(self.intTable.rowCount()):
            setNewLine = u''
            for j in range(self.intTable.columnCount()):
                item = self.intTable.item(i, j)
                if selectedItems:
                    if item in selectedItems:
                        toClipboard += item.text() + u'\t'
                        setNewLine = u'\n'
                elif item:
                    toClipboard += item.text() + u'\t'
                    setNewLine = u'\n'
            toClipboard += setNewLine
        QtWidgets.QApplication.clipboard().setText(toClipboard)

    def resultTableKeyPressEvent(self, event):
        if event.key() == QtCore.Qt.Key_C and event.modifiers() == QtCore.Qt.ControlModifier:
            self.copyToClipboard()

    def dataReceived(self, x, y, flux, y_norm):
        self.tx += 1
        self.intTable.setRowCount(self.tx)
        self.intTable.setItem(self.tx - 1, 0, QtWidgets.QTableWidgetItem(str(x)))
        self.intTable.setItem(self.tx - 1, 1, QtWidgets.QTableWidgetItem(str(y)))
        self.intTable.setItem(self.tx - 1, 2, QtWidgets.QTableWidgetItem(str(flux)))
        self.intTable.setItem(self.tx - 1, 3, QtWidgets.QTableWidgetItem(str(y_norm)))
        self.y.append(y)
        self.y_norm.append(y_norm)
        self.flux.append(flux)
        try:
            x = float(x)
        except ValueError:
            pass
        else:
            self.x.append(x)
            self.plotView.plot(self.x, self.y_norm, pen='g', clear=True)

    def saveSettings(self):
        settings = QtCore.QSettings()
        settings.setValue('WSleuth/Geometry', self.saveGeometry())
        settings.setValue('WSleuth/lastFolder', self.lastDir)
        settings.setValue('WSleuth/x1', self.x1SpinBox.value())
        settings.setValue('WSleuth/x2', self.x2SpinBox.value())
        settings.setValue('WSleuth/y1', self.y1SpinBox.value())
        settings.setValue('WSleuth/y2', self.y2SpinBox.value())
        settings.setValue('WSleuth/frame', self.frameSpinBox.value())
        settings.setValue('WSleuth/pm', self.pmSpinBox.value())
        settings.setValue('WSleuth/online', self.onlineCheckBox.isChecked())
        settings.setValue('WSleuth/folder', self.dirLineEdit.text())
        settings.setValue('WSleuth/re', self.reLineEdit.text())

    def loadSettings(self):
        settings = QtCore.QSettings()
        self.restoreGeometry(settings.value('WSleuth/Geometry', QtCore.QByteArray()))
        self.lastDir = settings.value('WSleuth/lastFolder', u'')
        self.dirLineEdit.setText(settings.value('WSleuth/folder'))
        self.x1SpinBox.setValue(float(settings.value('WSleuth/x1', 0)))
        self.x2SpinBox.setValue(float(settings.value('WSleuth/x2', 0)))
        self.y1SpinBox.setValue(float(settings.value('WSleuth/y1', 0)))
        self.y2SpinBox.setValue(float(settings.value('WSleuth/y2', 0)))
        self.frameSpinBox.setValue(float(settings.value('WSleuth/frame', 0)))
        self.pmSpinBox.setValue(float(settings.value('WSleuth/pm', 0)))
        self.onlineCheckBox.setChecked(pyqt2bool(settings.value('WSleuth/online')))
        self.reLineEdit.setText(settings.value('WSleuth/re', u''))

    def closeEvent(self, event):
        self.saveSettings()
        self.stopWalkSignal.emit()
        self.hide()
        self.walkThread.quit()
        self.walkThread.wait()

    @QtCore.pyqtSlot()
    def on_runButton_clicked(self):
        dire = self.dirLineEdit.text()
        if not dire or not os.path.exists(dire):
            return
        self.intTable.setColumnCount(4)
        self.intTable.setHorizontalHeaderLabels(['P', 'I', 'Flux', 'Inorm'])
        self.tx = 0
        self.stopButton.show()
        self.runButton.hide()
        self.walk(dire)

    @QtCore.pyqtSlot()
    def on_stopButton_clicked(self):
        self.stopButton.hide()
        self.runButton.show()
        self.stopWalkSignal.emit()

    @QtCore.pyqtSlot()
    def on_dirButton_clicked(self):
        # noinspection PyCallByClass,PyTypeChecker
        dire = QtWidgets.QFileDialog.getExistingDirectory(self, 'Select directory', self.lastDir)
        if not dire:
            return
        self.dirLineEdit.setText(dire)
        self.lastDir = dire

    @QtCore.pyqtSlot()
    def on_exitButton_clicked(self):
        self.close()

    def walk(self, dire):
        self.x, self.y, self.flux, self.y_norm = [], [], [], []
        x1 = self.x1SpinBox.value()
        x2 = self.x2SpinBox.value()
        y1 = self.y1SpinBox.value()
        y2 = self.y2SpinBox.value()
        frame = self.frameSpinBox.value()
        frame_pm = self.pmSpinBox.value()
        reg = self.reLineEdit.text()
        psi = self.onlineCheckBox.isChecked()
        self.worker = SleuthWalker(dire, frame, frame_pm, (x1, x2, y1, y2), reg, psi)
        self.worker.moveToThread(self.walkThread)
        # noinspection PyUnresolvedReferences
        self.walkThread.started.connect(self.worker.run)
        self.worker.finishedSignal.connect(self.on_stopButton_clicked)
        self.worker.finishedSignal.connect(self.walkThread.quit)
        self.worker.progressSignal.connect(self.dataReceived)
        self.stopWalkSignal.connect(self.worker.stop)
        self.walkThread.start()

    @QtCore.pyqtSlot()
    def on_saveButton_clicked(self):
        datName = QtWidgets.QFileDialog.getSaveFileName(self, 'Save file as a text', self.lastDir)[0]
        if not datName:
            return
        res = u''
        for i in range(self.intTable.rowCount()):
            for j in range(self.intTable.columnCount()):
                res += u'%s ' % self.intTable.item(i, j).text()
            res += u'\n'
        open(datName, 'w').write(res)

    @QtCore.pyqtSlot()
    def on_copyButton_clicked(self):
        self.intTable.selectAll()
        self.copyToClipboard()

    def keyPressEvent(self, event):
        key = event.key()
        if key == QtCore.Qt.Key_Escape:
            pass
        else:
            super().keyPressEvent(event)



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import glob
from PyQt5 import QtCore, QtGui, QtWidgets
from cryio import cbfimage
from .ui.ui_wheader import Ui_WHeader


class WHeader(QtWidgets.QDialog, Ui_WHeader):
    stopWorkerSignal = QtCore.pyqtSignal()
    runWorkerSignal = QtCore.pyqtSignal(str, list)

    def __init__(self, parent=None):
        QtWidgets.QDialog.__init__(self, parent)
        self._parent = parent
        self.setupUi(self)
        self.valuesList.addItems(sorted(list(cbfimage.ALL_KEYWORDS.keys()) + ['!File name', 'Date', 'Time']))
        self.loadSettings()
        self.stopButton.setVisible(False)
        self.createContextMenu()
        self.resultTable.keyPressEvent = self.resultTableKeyPressEvent
        self.resultTable.contextMenuEvent = self.resultTableContextMenuEvent
        self.workerThread = QtCore.QThread()
        self.worker = Worker()
        self.worker.moveToThread(self.workerThread)
        self.worker.nFilesSignal.connect(self.setResultTable)
        self.worker.finishedSignal.connect(self.on_stopButton_clicked)
        self.worker.progressSignal.connect(self.showProgress)
        self.stopWorkerSignal.connect(self.worker.stop)
        self.runWorkerSignal.connect(self.worker.run)
        self.workerThread.start()

    def createContextMenu(self):
        copyMenuAction = QtWidgets.QAction('&Copy', self.resultTable)
        copyMenuAction.setShortcut(QtGui.QKeySequence.Copy)
        copyMenuAction.triggered.connect(self.copyToClipboard)
        saveTextAction = QtWidgets.QAction('&Save as text', self.resultTable)
        saveTextAction.setShortcut(QtGui.QKeySequence.Save)
        saveTextAction.triggered.connect(self.on_saveButton_clicked)
        self.contextMenu = QtWidgets.QMenu()
        self.contextMenu.addAction(copyMenuAction)
        self.contextMenu.addAction(saveTextAction)

    def resultTableContextMenuEvent(self, event):
        self.contextMenu.exec_(event.globalPos())

    @QtCore.pyqtSlot()
    def copyToClipboard(self):
        selectedItems = self.resultTable.selectedItems()
        toClipboard = u''
        for i in range(self.resultTable.rowCount()):
            setNewLine = u''
            for j in range(self.resultTable.columnCount()):
                item = self.resultTable.item(i, j)
                if selectedItems:
                    if item in selectedItems:
                        toClipboard += item.text() + u'\t'
                        setNewLine = '\n'
                elif item:
                    toClipboard += item.text() + u'\t'
                    setNewLine = '\n'
            toClipboard += setNewLine
        # noinspection PyArgumentList
        QtWidgets.QApplication.clipboard().setText(toClipboard)

    def resultTableKeyPressEvent(self, event):
        if event.key() == QtCore.Qt.Key_C and event.modifiers() == QtCore.Qt.ControlModifier:
            self.copyToClipboard()

    def keyPressEvent(self, event):
        key = event.key()
        if key == QtCore.Qt.Key_Escape:
            pass
        else:
            QtWidgets.QDialog.keyPressEvent(self, event)

    def closeEvent(self, event):
        self.saveSettings()
        self.on_stopButton_clicked()
        self.hide()

    def checkedHeaderValues(self):
        checked = []
        for i in range(self.valuesList.count()):
            item = self.valuesList.item(i)
            if item.checkState():
                checked.append(item.text())
        return checked

    def saveSettings(self):
        settings = QtCore.QSettings()
        settings.setValue('WHeader/Geometry', self.saveGeometry())
        settings.setValue('WHeader/checked', self.checkedHeaderValues())
        settings.setValue('WHeader/lastFolder', self.lastFolder)

    def loadSettings(self):
        settings = QtCore.QSettings()
        self.restoreGeometry(settings.value('WHeader/Geometry', QtCore.QByteArray()))
        self.lastFolder = settings.value('WHeader/lastFolder', u'')
        self.folderLineEdit.setText(self.lastFolder)
        checked = settings.value('WHeader/checked') or []
        for i in range(self.valuesList.count()):
            item = self.valuesList.item(i)
            item.setFlags(QtCore.Qt.ItemIsUserCheckable | QtCore.Qt.ItemIsEnabled)
            item.setCheckState(QtCore.Qt.Checked if item.text() in checked else QtCore.Qt.Unchecked)
        self.on_valuesList_itemClicked()

    # noinspection PyUnusedLocal
    def on_valuesList_itemClicked(self, item=None):
        self.resultTable.clear()
        self.resultTable.setRowCount(0)
        checked = self.checkedHeaderValues()
        self.resultTable.setColumnCount(len(checked))
        self.resultTable.setHorizontalHeaderLabels(checked)

    @QtCore.pyqtSlot()
    def on_stopButton_clicked(self):
        self.runButton.setVisible(True)
        self.stopButton.setVisible(False)
        self.stopWorkerSignal.emit()

    @QtCore.pyqtSlot()
    def on_runButton_clicked(self):
        self.runButton.setVisible(False)
        self.stopButton.setVisible(True)
        self.on_valuesList_itemClicked()
        folder = self.folderLineEdit.text()
        checked = self.checkedHeaderValues()
        if folder and checked:
            self.runWorkerSignal.emit(folder, checked)

    @QtCore.pyqtSlot()
    def on_saveButton_clicked(self):
        # noinspection PyCallByClass
        datName = QtWidgets.QFileDialog.getSaveFileName(self, 'Save file as a text', self.lastFolder)[0]
        if not datName:
            return
        res = u'%s\n' % u' '.join(self.checkedHeaderValues())
        for i in range(self.resultTable.rowCount()):
            for j in range(self.resultTable.columnCount()):
                res += u'%s ' % self.resultTable.item(i, j).text()
            res += '\n'
        open(datName, 'w').write(res)

    def showProgress(self, i, full, values):
        for j, value in enumerate(values):
            item = QtWidgets.QTableWidgetItem(str(value))
            self.resultTable.setItem(i, j, item)
        self.runProgressBar.setValue(100 * (i + 1) / full)

    def setResultTable(self, n):
        self.resultTable.setRowCount(n)

    @QtCore.pyqtSlot()
    def on_folderButton_clicked(self):
        # noinspection PyCallByClass,PyTypeChecker,PyArgumentList
        folder = QtWidgets.QFileDialog.getExistingDirectory(self, 'Choose folder with cbf files', self.lastFolder)
        if folder:
            self.folderLineEdit.setText(folder)
            self.lastFolder = folder

    @QtCore.pyqtSlot()
    def on_copyButton_clicked(self):
        self.resultTable.selectAll()
        self.copyToClipboard()


class Worker(QtCore.QObject):
    nFilesSignal = QtCore.pyqtSignal(int)
    progressSignal = QtCore.pyqtSignal(int, int, list)
    finishedSignal = QtCore.pyqtSignal()

    def __init__(self):
        super().__init__()
        self.stopped = True

    def stop(self):
        self.stopped = True

    def run(self, folder, checked):
        if not os.path.exists(folder) or not os.path.isdir(folder):
            self.stopped = True
            return

        self.stopped = False
        files = glob.glob('{}/*.cbf'.format(folder))
        files.sort()
        self.nFilesSignal.emit(len(files))
        for i, cbf in enumerate(files):
            # noinspection PyArgumentList
            QtCore.QCoreApplication.processEvents()
            if self.stopped:
                return
            hdr = cbfimage.CbfHeader(cbf)
            values = []
            for key in checked:
                try:
                    values.append(hdr[key])
                except KeyError:
                    continue
            self.progressSignal.emit(i, len(files), values)
        self.finishedSignal.emit()
        self.stopped = True



# -*- coding: utf-8 -*-


from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_WSnblToolBox(object):
    def setupUi(self, WSnblToolBox):
        WSnblToolBox.setObjectName("WSnblToolBox")
        WSnblToolBox.resize(612, 276)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/swiss"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        WSnblToolBox.setWindowIcon(icon)
        WSnblToolBox.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.gridLayout = QtWidgets.QGridLayout(WSnblToolBox)
        self.gridLayout.setObjectName("gridLayout")
        self.crysisButton = QtWidgets.QToolButton(WSnblToolBox)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/crysis"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.crysisButton.setIcon(icon1)
        self.crysisButton.setIconSize(QtCore.QSize(72, 72))
        self.crysisButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.crysisButton.setObjectName("crysisButton")
        self.gridLayout.addWidget(self.crysisButton, 0, 0, 1, 1)
        self.darthButton = QtWidgets.QToolButton(WSnblToolBox)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/darth"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.darthButton.setIcon(icon2)
        self.darthButton.setIconSize(QtCore.QSize(72, 72))
        self.darthButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.darthButton.setObjectName("darthButton")
        self.gridLayout.addWidget(self.darthButton, 0, 1, 1, 1)
        self.convertButton = QtWidgets.QToolButton(WSnblToolBox)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/convert"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.convertButton.setIcon(icon3)
        self.convertButton.setIconSize(QtCore.QSize(72, 72))
        self.convertButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.convertButton.setObjectName("convertButton")
        self.gridLayout.addWidget(self.convertButton, 0, 2, 1, 1)
        self.sigmasButton = QtWidgets.QToolButton(WSnblToolBox)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/scale"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.sigmasButton.setIcon(icon4)
        self.sigmasButton.setIconSize(QtCore.QSize(72, 72))
        self.sigmasButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.sigmasButton.setObjectName("sigmasButton")
        self.gridLayout.addWidget(self.sigmasButton, 0, 3, 1, 1)
        self.headExButton = QtWidgets.QToolButton(WSnblToolBox)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/header"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.headExButton.setIcon(icon5)
        self.headExButton.setIconSize(QtCore.QSize(72, 72))
        self.headExButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.headExButton.setObjectName("headExButton")
        self.gridLayout.addWidget(self.headExButton, 1, 0, 1, 1)
        self.sleuthButton = QtWidgets.QToolButton(WSnblToolBox)
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/sleuth"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.sleuthButton.setIcon(icon6)
        self.sleuthButton.setIconSize(QtCore.QSize(72, 72))
        self.sleuthButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.sleuthButton.setObjectName("sleuthButton")
        self.gridLayout.addWidget(self.sleuthButton, 1, 1, 1, 1)
        self.roerikButton = QtWidgets.QToolButton(WSnblToolBox)
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/roerik"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.roerikButton.setIcon(icon7)
        self.roerikButton.setIconSize(QtCore.QSize(72, 72))
        self.roerikButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.roerikButton.setObjectName("roerikButton")
        self.gridLayout.addWidget(self.roerikButton, 1, 2, 1, 1)
        self.infoButton = QtWidgets.QToolButton(WSnblToolBox)
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/info"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.infoButton.setIcon(icon8)
        self.infoButton.setIconSize(QtCore.QSize(72, 72))
        self.infoButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.infoButton.setObjectName("infoButton")
        self.gridLayout.addWidget(self.infoButton, 1, 3, 1, 1)

        self.retranslateUi(WSnblToolBox)
        QtCore.QMetaObject.connectSlotsByName(WSnblToolBox)

    def retranslateUi(self, WSnblToolBox):
        _translate = QtCore.QCoreApplication.translate
        WSnblToolBox.setWindowTitle(_translate("WSnblToolBox", "SNBL Toolbox"))
        self.crysisButton.setToolTip(_translate("WSnblToolBox", "Crysis produces the need files for crysalis"))
        self.crysisButton.setStatusTip(_translate("WSnblToolBox", "Crysis produces the need files for crysalis"))
        self.crysisButton.setWhatsThis(_translate("WSnblToolBox", "Crysis produces the need files for crysalis"))
        self.crysisButton.setText(_translate("WSnblToolBox", "Crysis"))
        self.darthButton.setToolTip(_translate("WSnblToolBox", "Generate CCP4 file from a crysalis project"))
        self.darthButton.setStatusTip(_translate("WSnblToolBox", "Generate CCP4 file from a crysalis project"))
        self.darthButton.setWhatsThis(_translate("WSnblToolBox", "Generate CCP4 file from a crysalis project"))
        self.darthButton.setText(_translate("WSnblToolBox", "3DV"))
        self.convertButton.setToolTip(_translate("WSnblToolBox", "Convert the pilatus file to the other formats"))
        self.convertButton.setStatusTip(_translate("WSnblToolBox", "Convert the pilatus file to the other formats"))
        self.convertButton.setWhatsThis(_translate("WSnblToolBox", "Convert the pilatus file to the other formats"))
        self.convertButton.setText(_translate("WSnblToolBox", "Converter"))
        self.sigmasButton.setToolTip(_translate("WSnblToolBox", "Scale data from fit2d"))
        self.sigmasButton.setStatusTip(_translate("WSnblToolBox", "Scale data from fit2d"))
        self.sigmasButton.setWhatsThis(_translate("WSnblToolBox", "Scale data from fit2d"))
        self.sigmasButton.setText(_translate("WSnblToolBox", "σ-scaler"))
        self.headExButton.setToolTip(_translate("WSnblToolBox", "Extract information from cbf headers"))
        self.headExButton.setStatusTip(_translate("WSnblToolBox", "Extract information from cbf headers"))
        self.headExButton.setWhatsThis(_translate("WSnblToolBox", "Extract information from cbf headers"))
        self.headExButton.setText(_translate("WSnblToolBox", "HeadEx"))
        self.sleuthButton.setToolTip(_translate("WSnblToolBox", "Search intensity through the folder"))
        self.sleuthButton.setStatusTip(_translate("WSnblToolBox", "Search intensity through the folder"))
        self.sleuthButton.setWhatsThis(_translate("WSnblToolBox", "Search intensity through the folder"))
        self.sleuthButton.setText(_translate("WSnblToolBox", "Sleuth"))
        self.roerikButton.setToolTip(_translate("WSnblToolBox", "Calculate possible beam scattering"))
        self.roerikButton.setStatusTip(_translate("WSnblToolBox", "Calculate possible beam scattering"))
        self.roerikButton.setWhatsThis(_translate("WSnblToolBox", "Calculate possible beam scattering"))
        self.roerikButton.setText(_translate("WSnblToolBox", "Hrœrekr"))
        self.infoButton.setToolTip(_translate("WSnblToolBox", "Information"))
        self.infoButton.setStatusTip(_translate("WSnblToolBox", "Information"))
        self.infoButton.setWhatsThis(_translate("WSnblToolBox", "Information"))
        self.infoButton.setText(_translate("WSnblToolBox", "Info"))

from . import resources_rc



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import numpy as np
from PyQt5 import QtGui, QtCore, QtWidgets
from cryio.parparser import ParParser
from ..ui.ui_wgunnar import Ui_WGunnar
from . import neighbors


class WGunnar(QtWidgets.QDialog, Ui_WGunnar):
    def __init__(self, parent):
        super(WGunnar, self).__init__(parent)
        self.setupUi(self)
        self._parent = parent
        doubbleValidator = QtGui.QDoubleValidator()
        for name, obj in self.interEdit():
            obj.setValidator(doubbleValidator)
        self.loadSettings()
        self.resTable.horizontalHeader().setResizeMode(QtWidgets.QHeaderView.ResizeToContents)
        self.resTable.verticalHeader().setResizeMode(QtWidgets.QHeaderView.ResizeToContents)

    def saveSettings(self):
        s = QtCore.QSettings()
        s.setValue('WRoerik/Geometry', self.saveGeometry())
        for name, obj in self.interEdit():
            s.setValue('WRoerik/{}'.format(name), obj.text())
        s.setValue('WRoerik/ubm', self.ubmText.toPlainText())
        s.setValue('WRoerik/folder', self.folder)

    def loadSettings(self):
        s = QtCore.QSettings()
        self.restoreGeometry(s.value('WRoerik/Geometry', QtCore.QByteArray()))
        for name, obj in self.interEdit():
            obj.setText(s.value('WRoerik/{}'.format(name), '0'))
        self.ubmText.setPlainText(s.value('WRoerik/ubm', ''))
        self.folder = s.value('WRoerik/folder', '')

    def closeEvent(self, event):
        self.saveSettings()
        self.hide()

    def interEdit(self):
        for name in self.__dict__:
            if name.endswith('Edit'):
                yield name, self.__dict__[name]

    @QtCore.pyqtSlot()
    def on_parButton_clicked(self):
        # noinspection PyCallByClass,PyTypeChecker
        par = QtWidgets.QFileDialog.getOpenFileName(self, 'Open .par file', self.folder, 'Crysalis par (*.par)')[0]
        if par:
            self.folder = os.path.dirname(par)
            self.parsePar(par)

    def parsePar(self, par):
        par = ParParser(par)
        self.alphaEdit.setText(str(par.alpha))
        self.betaEdit.setText(str(par.beta))
        self.lambdaEdit.setText(str(par.wavelength))
        self.d0Edit.setText(str(par.dist))
        self.phixEdit.setText(str(par.phix))
        self.phiyEdit.setText(str(par.phiy))
        self.ubmText.setPlainText(' '.join([str(i) for i in par.ub]))
        self.x0Edit.setText(str(par.xc))
        self.y0Edit.setText(str(par.yc))

    # noinspection PyArgumentList
    @QtCore.pyqtSlot(str, name='on_alphaEdit_textChanged')
    @QtCore.pyqtSlot(str, name='on_betaEdit_textChanged')
    @QtCore.pyqtSlot(str, name='on_lambdaEdit_textChanged')
    @QtCore.pyqtSlot(str, name='on_d0Edit_textChanged')
    @QtCore.pyqtSlot(str, name='on_phixEdit_textChanged')
    @QtCore.pyqtSlot(str, name='on_phiyEdit_textChanged')
    @QtCore.pyqtSlot(str, name='on_x0Edit_textChanged')
    @QtCore.pyqtSlot(str, name='on_y0Edit_textChanged')
    @QtCore.pyqtSlot(name='on_ubmText_textChanged')
    @QtCore.pyqtSlot(str, name='on_y0Edit_textChanged')
    @QtCore.pyqtSlot(str, name='on_deltaEdit_textChanged')
    @QtCore.pyqtSlot(str, name='on_omegaEdit_textChanged')
    @QtCore.pyqtSlot(str, name='on_kappaEdit_textChanged')
    @QtCore.pyqtSlot(str, name='on_phiEdit_textChanged')
    def calculateVectors(self, text=None):
        self.resTable.clear()
        self.resTable.setRowCount(3)
        self.resTable.setHorizontalHeaderLabels(['(h,k,l)', '(x,y)', 'alpha_h'])
        try:
            ubm = np.fromstring(self.ubmText.toPlainText(), dtype=float, count=9, sep=' ').reshape((3, 3))
            wavelength = float(self.lambdaEdit.text())
            delta = float(self.deltaEdit.text())
            alpha = float(self.alphaEdit.text())
            beta = float(self.betaEdit.text())
            omega = float(self.omegaEdit.text())
            kappa = float(self.kappaEdit.text())
            phi = float(self.phiEdit.text())
            phix = float(self.phixEdit.text())
            phiy = float(self.phiyEdit.text())
            y0 = float(self.x0Edit.text())
            x0 = float(self.y0Edit.text())
            d0 = float(self.d0Edit.text())
        except ValueError:
            return
        res = neighbors.neighbors(ubm, wavelength, delta, alpha, beta, omega, kappa, phi, phix, phiy, x0, y0, d0)
        if not res:
            return
        self.resTable.setRowCount(len(res))
        for i, item in enumerate(res):
            for j, value in enumerate(item):
                self.resTable.setItem(i, j, QtWidgets.QTableWidgetItem(str(value)))

    def keyPressEvent(self, event):
        key = event.key()
        if key == QtCore.Qt.Key_Escape:
            pass
        else:
            QtWidgets.QDialog.keyPressEvent(self, event)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
from glob import glob
import subprocess
from setuptools import setup

dirname = 'SNBLToolBox'
file_frozen = '{}/frozen.py'.format(dirname)
we_run_setup = False
if not os.path.exists(file_frozen):
    we_run_setup = True
    hash_ = subprocess.Popen(['hg', 'id', '-i'], stdout=subprocess.PIPE).stdout.read().decode().strip()
    print('SNBL Toolbox mercurial hash is {}'.format(hash_))
    frozen = open(file_frozen, 'w')
    frozen.write('hg_hash = "{}"'.format(hash_))
    frozen.close()

modules = glob('{}/ui/*.py'.format(dirname))
try:
    modules.remove('{}/ui/compile.py'.format(dirname))
except ValueError:
    pass
modules += glob('{}/*.py'.format(dirname))
modules += glob('{}/roerik/*.py'.format(dirname))
modules = [s.replace('/', '.').split('.py')[0] for s in modules]

print(modules)
setup(
    name='snbl-toolbox',
    version='2017.8.11',
    description='SNBL Toolbox',
    author='Vadim Dyadkin',
    author_email='dyadkin@gmail.com',
    url='https://hg.3lp.cx/snbltb',
    license='GPLv3',
    install_requires=[
        'numpy',
        'cryio',
        'pyqtgraph>=0.10.0',
        'crymon',
    ],
    include_package_data=True,
    entry_points={
        'gui_scripts': [
            'snbl={}.__init__:main'.format(dirname),
        ],
    },
    py_modules=modules,
)

if we_run_setup:
    os.remove(file_frozen)



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import re
import glob
import sip
from PyQt5 import QtCore, QtGui, QtWidgets
import cryio
from cryio.cbfimage import CbfImage
from . import pyqt2bool
# noinspection PyUnresolvedReferences
from .ui import resources_rc


NEGATIVE_THRESHOLD = -3


class Page(QtWidgets.QWizardPage):
    def __init__(self, parent=None, prevPage=None):
        QtWidgets.QWizardPage.__init__(self, parent)
        self.prevPage = prevPage
        self.setupUi()

    def setupUi(self):
        pass


class FilesPage(Page):
    def setupUi(self):
        folderButton = QtWidgets.QPushButton('Choose folder...')
        folderButton.clicked.connect(self.chooseFolder)

        filesButton = QtWidgets.QPushButton('Choose files...')
        filesButton.clicked.connect(self.chooseFiles)

        label = QtWidgets.QLabel('or')

        hlayout = QtWidgets.QHBoxLayout()
        hlayout.addWidget(folderButton)
        hlayout.addWidget(label)
        hlayout.addWidget(filesButton)
        hlayout.addStretch()

        self.fileList = QtWidgets.QListWidget()
        self.fileList.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)

        delButton = QtWidgets.QPushButton('Delete items')
        delButton.clicked.connect(self.delItems)

        clearButton = QtWidgets.QPushButton('Clear')
        clearButton.clicked.connect(self.clearItems)

        self.label = QtWidgets.QLabel('Selected files: 0')

        hlayout1 = QtWidgets.QHBoxLayout()
        hlayout1.addWidget(self.label)
        hlayout1.addStretch()
        hlayout1.addWidget(delButton)
        hlayout1.addWidget(clearButton)

        vlayout = QtWidgets.QVBoxLayout()
        vlayout.addLayout(hlayout)
        vlayout.addWidget(self.fileList)
        vlayout.addLayout(hlayout1)
        self.setLayout(vlayout)

    def initializePage(self):
        settings = QtCore.QSettings()
        self.lastFolder = settings.value('Convert/lastFolder', '')
        self.files = {}

    @QtCore.pyqtSlot()
    def chooseFolder(self, what='*.cbf *.edf'):
        d = QtWidgets.QFileDialog.getExistingDirectory(self, 'Choose the folder with %s files' % what, self.lastFolder)
        if not d:
            return
        self.lastFolder = d
        pd = QtWidgets.QProgressDialog('Reading directories...', 'Cancel', 0, 1, self)
        pd.setAutoClose(False)
        pd.setAutoReset(False)
        pd.setWindowModality(QtCore.Qt.WindowModal)
        counter, files = 1, []
        for dirpath, dirnames, filenames in os.walk(d):
            pd.setMaximum(counter + len(dirnames))
            pd.setValue(counter)
            pd.setLabelText('Reading {}'.format(dirpath))
            counter += len(dirnames)
            if pd.wasCanceled():
                self.files = {}
                return
            QtCore.QCoreApplication.processEvents()
            self.files[dirpath] = []
            for ext in what.split(' '):
                fs = glob.glob(os.path.join(dirpath, ext))
                self.files[dirpath] += fs
                files += fs
        pd.close()
        self.openFiles(files)

    @QtCore.pyqtSlot()
    def chooseFiles(self, what='*.cbf *.edf'):
        files = QtWidgets.QFileDialog.getOpenFileNames(self, 'Choose %s files' % what,
                                                       self.lastFolder, 'Pilatus data (%s)' % what)[0]
        if files:
            self.files[os.path.dirname(files[0])] = files
        self.openFiles(files)

    def openFiles(self, files):
        if not files:
            return
        self.fileList.addItems(files)
        self.label.setText('Selected files: %d' % self.fileList.count())

    @QtCore.pyqtSlot()
    def delItems(self):
        for item in self.fileList.selectedItems():
            sip.delete(item)
        self.label.setText('Selected files: %d' % self.fileList.count())

    @QtCore.pyqtSlot()
    def clearItems(self):
        self.fileList.clear()
        self.label.setText('Selected files: %d' % self.fileList.count())

    def validatePage(self):
        settings = QtCore.QSettings()
        settings.setValue('Convert/lastFolder', self.lastFolder)
        return Page.validatePage(self)


class OptionsPage(Page):
    def setupUi(self):
        self.cbf = QtWidgets.QRadioButton('cbf')
        self.cbf.setChecked(True)
        self.edf = QtWidgets.QRadioButton('edf')

        hrlayout = QtWidgets.QHBoxLayout()
        vrlayout1 = QtWidgets.QVBoxLayout()
        vrlayout1.addWidget(self.cbf)
        vrlayout2 = QtWidgets.QVBoxLayout()
        vrlayout2.addWidget(self.edf)
        hrlayout.addLayout(vrlayout1)
        hrlayout.addLayout(vrlayout2)

        radiosGroupBox = QtWidgets.QGroupBox('Choose the format to convert')
        radiosGroupBox.setLayout(hrlayout)

        self.binningCheckBox = QtWidgets.QCheckBox('Make binning')
        self.binningEveryRadio = QtWidgets.QRadioButton('for every')
        self.binningSpinBox = QtWidgets.QSpinBox()
        self.binningSpinBox.setDisabled(True)
        self.binningSpinBox.setMinimum(2)
        self.binningSpinBox.setMaximum(65535)
        self.binningPeriodRadio = QtWidgets.QRadioButton('periodically')
        filesLabel = QtWidgets.QLabel('files')
        self.binningGroup = QtWidgets.QButtonGroup()
        self.binningGroup.addButton(self.binningEveryRadio)
        self.binningGroup.addButton(self.binningPeriodRadio)

        def toggleBinning(enable=None):
            if enable is None:
                enable = self.binningCheckBox.isChecked()
            self.binningSpinBox.setEnabled(enable)
            self.binningEveryRadio.setEnabled(enable)
            self.binningPeriodRadio.setEnabled(enable)
            filesLabel.setEnabled(enable)

        toggleBinning(False)
        self.binningEveryRadio.setChecked(True)
        self.binningCheckBox.toggled.connect(toggleBinning)
        self.binningPeriodRadio.toggled.connect(
            lambda: self.binningSpinBox.setDisabled(self.binningPeriodRadio.isChecked())
        )

        hlayout1 = QtWidgets.QHBoxLayout()
        hlayout1.addWidget(self.binningCheckBox)
        hlayout1.addWidget(self.binningEveryRadio)
        hlayout1.addWidget(self.binningSpinBox)
        hlayout1.addWidget(filesLabel)
        hlayout1.addWidget(self.binningPeriodRadio)
        hlayout1.addStretch()

        self.negativeCheckBox = QtWidgets.QCheckBox('Set negative intensity as')
        self.negativeSpinBox = QtWidgets.QSpinBox()
        self.negativeSpinBox.setMinimum(-2)
        self.negativeSpinBox.setDisabled(True)
        self.negativeCheckBox.clicked.connect(
            lambda: self.negativeSpinBox.setEnabled(self.negativeCheckBox.isChecked())
        )

        hlayout2 = QtWidgets.QHBoxLayout()
        hlayout2.addWidget(self.negativeCheckBox)
        hlayout2.addWidget(self.negativeSpinBox)
        hlayout2.addStretch()

        self.moveAngleCheckBox = QtWidgets.QCheckBox('Move final angles:')
        self.startingAngleSpinBox = QtWidgets.QDoubleSpinBox()
        self.startingAngleSpinBox.setDecimals(4)
        self.startingAngleSpinBox.setMinimum(-360)
        self.startingAngleSpinBox.setMaximum(360)
        self.angleIncrementSpinBox = QtWidgets.QDoubleSpinBox()
        self.angleIncrementSpinBox.setDecimals(4)
        self.angleIncrementSpinBox.setMinimum(-360)
        self.angleIncrementSpinBox.setMaximum(360)
        startLabel = QtWidgets.QLabel('start')
        incLabel = QtWidgets.QLabel('increment')
        hlayout3 = QtWidgets.QHBoxLayout()
        hlayout3.addWidget(self.moveAngleCheckBox)
        hlayout3.addWidget(startLabel)
        hlayout3.addWidget(self.startingAngleSpinBox)
        hlayout3.addWidget(incLabel)
        hlayout3.addWidget(self.angleIncrementSpinBox)
        hlayout3.addStretch()

        def showAngle(show):
            self.moveAngleCheckBox.setVisible(show)
            self.startingAngleSpinBox.setVisible(show)
            self.angleIncrementSpinBox.setVisible(show)
            startLabel.setVisible(show)
            incLabel.setVisible(show)

        showAngle(True)
        self.cbf.toggled[bool].connect(showAngle)

        def enableAngle(en):
            self.angleIncrementSpinBox.setEnabled(en)
            self.startingAngleSpinBox.setEnabled(en)

        enableAngle(False)
        self.moveAngleCheckBox.toggled[bool].connect(enableAngle)

        self.int32 = QtWidgets.QRadioButton('Int32 (for fit2d)')
        self.int32.setChecked(True)
        self.int16 = QtWidgets.QRadioButton('Int16')
        self.intGroup = QtWidgets.QButtonGroup()
        self.intGroup.addButton(self.int32)
        self.intGroup.addButton(self.int16)
        self.flipCheckBox = QtWidgets.QCheckBox('Flip image (for fit2d)')
        self.flipCheckBox.setChecked(True)
        self.addHeaderCheckBox = QtWidgets.QCheckBox('Add edf header (for fit2d)')
        self.addHeaderCheckBox.setChecked(True)
        hlayout4 = QtWidgets.QHBoxLayout()
        hlayout4.addWidget(self.int32)
        hlayout4.addWidget(self.int16)
        hlayout4.addStretch()
        hlayout5 = QtWidgets.QHBoxLayout()
        hlayout5.addWidget(self.flipCheckBox)
        hlayout5.addStretch()
        hlayout6 = QtWidgets.QHBoxLayout()
        hlayout6.addWidget(self.addHeaderCheckBox)
        hlayout6.addStretch()

        def showEdfOpts(show):
            self.int16.setVisible(show)
            self.int32.setVisible(show)
            self.flipCheckBox.setVisible(show)
            self.addHeaderCheckBox.setVisible(show)

        showEdfOpts(False)
        self.edf.toggled[bool].connect(showEdfOpts)

        vlayout1 = QtWidgets.QVBoxLayout()
        vlayout1.addLayout(hlayout1)
        vlayout1.addLayout(hlayout2)
        vlayout1.addLayout(hlayout3)
        vlayout1.addLayout(hlayout4)
        vlayout1.addLayout(hlayout5)
        vlayout1.addLayout(hlayout6)

        optionsGroupBox = QtWidgets.QGroupBox('Options')
        optionsGroupBox.setLayout(vlayout1)

        vlayout2 = QtWidgets.QVBoxLayout()
        vlayout2.addWidget(radiosGroupBox)
        vlayout2.addWidget(optionsGroupBox)
        self.setLayout(vlayout2)

    def initializePage(self):
        settings = QtCore.QSettings()
        self.setRadio(settings.value('Convert/format', 'cbf'), 'edf', 'cbf')
        binning = pyqt2bool(settings.value('Convert/binning', False))
        self.binningCheckBox.setChecked(binning)
        self.binningSpinBox.setValue(int(settings.value('Convert/binning_value', 1)))
        self.setRadio(settings.value('Convert/bintype', 'binningEveryRadio'), 'binningEveryRadio', 'binningPeriodRadio')
        negative = pyqt2bool(settings.value('Convert/negative', False))
        self.negativeCheckBox.setChecked(negative)
        self.negativeSpinBox.setEnabled(negative)
        self.negativeSpinBox.setValue(int(settings.value('Convert/negative_value', 0)))
        self.angleIncrementSpinBox.setValue(float(settings.value('Convert/angleIncrement', 0)))
        self.startingAngleSpinBox.setValue(float(settings.value('Convert/startingAngle', 0)))
        self.flipCheckBox.setChecked(pyqt2bool(settings.value('Convert/edfFlip', True)))
        self.setRadio(settings.value('Convert/edfInt', 'int32'), 'int16', 'int32')
        self.addHeaderCheckBox.setChecked(pyqt2bool(settings.value('Convert/edfAddHeader', True)))
        move = pyqt2bool(settings.value('Convert/move', False))
        self.moveAngleCheckBox.setChecked(move)

    def getRadio(self, *radios):
        for item in radios:
            if self.__dict__[item].isChecked():
                return item

    def setRadio(self, value, *radios):
        for item in radios:
            if value == item:
                self.__dict__[item].setChecked(True)
                return

    def validatePage(self):
        settings = QtCore.QSettings()
        settings.setValue('Convert/format', self.getRadio('cbf', 'edf'))
        settings.setValue('Convert/binning', self.binningCheckBox.isChecked())
        settings.setValue('Convert/bintype', self.getRadio('binningEveryRadio', 'binningPeriodRadio'))
        settings.setValue('Convert/binning_value', self.binningSpinBox.value())
        settings.setValue('Convert/negative', self.negativeCheckBox.isChecked())
        settings.setValue('Convert/negative_value', self.negativeSpinBox.value())
        settings.setValue('Convert/startingAngle', self.startingAngleSpinBox.value())
        settings.setValue('Convert/angleIncrement', self.angleIncrementSpinBox.value())
        settings.setValue('Convert/edfInt', self.getRadio('int16', 'int32'))
        settings.setValue('Convert/edfFlip', self.flipCheckBox.isChecked())
        settings.setValue('Convert/edfAddHeader', self.addHeaderCheckBox.isChecked())
        settings.setValue('Convert/move', self.moveAngleCheckBox.isChecked())
        return Page.validatePage(self)


class ConvertPage(Page):
    def setupUi(self):
        self.progress = QtWidgets.QProgressBar()
        self.progress.setMaximum(100)
        self.log = QtWidgets.QListWidget()

        layout = QtWidgets.QVBoxLayout()
        layout.addWidget(self.progress)
        layout.addWidget(self.log)
        self.setLayout(layout)

    def isComplete(self):
        return self.wizard().stopped


class ConvertWizard(QtWidgets.QWizard):
    terminateSignal = QtCore.pyqtSignal()

    def keyPressEvent(self, event):
        key = event.key()
        if key == QtCore.Qt.Key_Escape:
            pass
        else:
            QtWidgets.QDialog.keyPressEvent(self, event)

    def __init__(self, parent=None):
        QtWidgets.QWizard.__init__(self, parent)
        self.addPage(FilesPage())
        self.addPage(OptionsPage())
        self.addPage(ConvertPage())
        self.setWindowTitle('Pylatus image converter')
        self.setOptions(QtWidgets.QWizard.NoBackButtonOnLastPage)
        self.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        matrix = QtGui.QTransform().rotate(-90)
        pixmap = QtGui.QPixmap(':/snbl').scaled(281, 151, QtCore.Qt.KeepAspectRatio,
                                                QtCore.Qt.SmoothTransformation).transformed(matrix)
        self.setPixmap(QtWidgets.QWizard.WatermarkPixmap, pixmap)
        self.loadSettings()
        self.setWindowIcon(QtGui.QIcon(':/convert'))
        self.currentIdChanged[int].connect(self.checkPage)
        self.stopped = False
        self.workerThread = QtCore.QThread()
        self.rejected.connect(self.terminate)

    def terminate(self):
        self.terminateSignal.emit()
        self.finished()

    def checkPage(self, page):
        if page == 2:
            self.convert()

    def closeEvent(self, event):
        self.saveSettings()
        self.terminate()
        event.accept()

    def loadSettings(self):
        settings = QtCore.QSettings()
        self.restoreGeometry(settings.value('Convert/Geometry', QtCore.QByteArray()))

    def saveSettings(self):
        settings = QtCore.QSettings()
        settings.setValue('Convert/Geometry', self.saveGeometry())

    def showProgress(self, images, converted, queueLength):
        self.log.addItem(images)
        self.log.scrollToBottom()
        self.progress.setValue(100.0 * converted / queueLength)

    def finished(self, p_int=0):
        self.stopped = True
        if hasattr(self, 'page2'):
            self.page2.completeChanged.emit()
        self.workerThread.quit()
        self.workerThread.wait()

    def convert(self):
        self.page2 = self.page(2)
        self.progress = self.page2.progress
        self.log = self.page2.log

        page1 = self.page(1)
        if page1.negativeCheckBox.isChecked():
            negative = page1.negativeSpinBox.value()
        else:
            negative = NEGATIVE_THRESHOLD

        self.format = page1.getRadio('cbf', 'edf')
        if page1.moveAngleCheckBox.isChecked():
            move = [page1.startingAngleSpinBox.value(), page1.angleIncrementSpinBox.value()]
        else:
            move = []

        images = self.page(0).files
        binning = page1.binningCheckBox.isChecked()
        binnum = page1.binningSpinBox.value() if page1.binningSpinBox.isEnabled() else 0
        opts = {
            'format': self.format,
            'negative': negative,
            'move': move,
            'edfInt': page1.getRadio('int16', 'int32'),
            'edfFlip': page1.flipCheckBox.isChecked(),
            'edfHeader': page1.addHeaderCheckBox.isChecked(),
        }
        self.workerManager = CbfConverterManager(images, binning, binnum, opts)
        self.workerManager.moveToThread(self.workerThread)
        self.workerThread.started.connect(self.workerManager.convert)
        self.workerManager.progressSignal.connect(self.showProgress)
        self.workerManager.finishedSignal.connect(self.workerFinished)
        self.terminateSignal.connect(self.workerManager.terminateAll)
        self.workerThread.start()

    def workerFinished(self, errors):
        if errors:
            QtWidgets.QMessageBox.critical(self, 'Errors', '\n'.join(e for e in errors))
        self.finished()


class Signals(QtCore.QObject):
    progressSignal = QtCore.pyqtSignal(str)
    errorSignal = QtCore.pyqtSignal(str)
    finishedSignal = QtCore.pyqtSignal(object)
    stopSignal = QtCore.pyqtSignal()


class CbfConverterWorker(QtCore.QRunnable):
    def __init__(self, opts):
        super().__init__()
        self.opts = opts
        self.stopped = False
        self.signals = Signals()
        self.signals.stopSignal.connect(self.stop)

    def run(self):
        try:
            self.convert()
        except Exception as e:
            self.signals.errorSignal.emit(str(e))
        self.stopped = True
        self.signals.finishedSignal.emit(self)

    def stop(self):
        self.stopped = True

    def convert(self):
        if self.stopped:
            return
        folder = os.path.join(os.path.dirname(self.opts['images'][0]), self.opts['format'])
        try:
            os.mkdir(folder)
        except FileExistsError:
            pass
        img = cryio.openImage(self.opts['images'][0])
        oi = img.header.get('Omega_increment')
        pi = img.header.get('Phi_increment')
        ai = img.header.get('Angle_increment')
        for image in self.opts['images'][1:]:
            if self.stopped:
                return
            img += cryio.openImage(image)
        if self.opts['negative'] > NEGATIVE_THRESHOLD:
            img.array[img.array < 0] = self.opts['negative']
        if self.opts['move'] and isinstance(img, CbfImage):
            img.header['Start_angle'], img.header['Angle_increment'] = self.opts['move']
            if 'Oscillation_axis' in img.header:
                axis = img.header['Oscillation_axis'].upper()
                if axis == 'OMEGA':
                    img.header['Omega'], img.header['Omega_increment'] = self.opts['move']
                elif axis == 'PHI':
                    img.header['Phi'], img.header['Phi_increment'] = self.opts['move']
        if self.opts['binperiods']:
            if oi is not None:
                img.header['Omega_increment'] = oi
            if pi is not None:
                img.header['Phi_increment'] = pi
            if ai is not None:
                img.header['Angle_increment'] = ai
        self.opts.update(img.header)
        newImg = os.path.join(folder, self.opts['name'])
        img.save(newImg, self.opts['format'], **self.opts)
        self.signals.progressSignal.emit('{}'.format(newImg))


class CbfConverterManager(QtCore.QObject):
    progressSignal = QtCore.pyqtSignal(str, int, int)
    finishedSignal = QtCore.pyqtSignal(list)

    def __init__(self, images, binning, binnum, opts):
        super().__init__()
        self.stopped = True
        self.images = images
        self.opts = opts
        self.binning = binning
        self.binnum = binnum
        self.binperiods = self.binnum == 0
        self.opts['binperiods'] = self.binperiods
        self.pool = QtCore.QThreadPool.globalInstance()
        self.workers, self.errors = [], []
        self.startAngle = None
        self.angleInc = None

    def terminateAll(self):
        self.stopped = True
        for worker in self.workers:
            worker.signals.stopSignal.emit()
        self.workers = []

    def simpleConversion(self):
        for dirpath in self.images:
            for cbf in self.images[dirpath]:
                name = '{}.{[format]}'.format(os.path.basename(cbf)[:-4], self.opts)
                args = self.opts.copy()
                args.update({'images': (cbf,), 'name': name})
                yield self.moveAngle(args)

    def moveAngle(self, args):
        if args['move']:
            if self.startAngle is None and self.angleInc is None:
                self.startAngle, self.angleInc = args['move']
            else:
                self.startAngle += self.angleInc
                args['move'] = self.startAngle, self.angleInc
        return args

    def makeChunk(self, i, images):
        args = self.opts.copy()
        name = '{}_{:05d}.{[format]}'.format('_'.join(os.path.basename(images[0])[:-4].split('_')[:-1]), i, self.opts)
        args.update({'images': images, 'name': name})
        return self.moveAngle(args)

    def sort_periods(self, name):
        pattern = re.compile(r'.+_(\d+)p_(\d+)\.cbf$')
        try:
            period, frame = [int(i) for i in pattern.split(name)[1:3]]
        except IndexError:
            return name
        if period > self.binnum:
            self.binnum = period
        return frame, period

    def binFrames(self):
        for dirpath in self.images:
            self.startAngle = None
            self.angleInc = None
            i = 0
            images = []
            if self.binperiods:
                self.binnum = 0
            self.images[dirpath].sort(key=self.sort_periods if self.binperiods else None)
            for cbf in self.images[dirpath]:
                images.append(cbf)
                if len(images) == self.binnum:
                    yield self.makeChunk(i, images)
                    images = []
                    i += 1
            converted = i * self.binnum
            if converted < len(self.images[dirpath]):
                yield self.makeChunk(i, self.images[dirpath][converted:])

    def convert(self):
        self.stopped = False
        self.chunks = self.chunkDone = 0
        if self.binning:
            images = self.binFrames()
        else:
            images = self.simpleConversion()
        for chunk in images:
            QtCore.QCoreApplication.processEvents()
            if self.stopped:
                return
            self.chunks += 1
            worker = CbfConverterWorker(chunk)
            worker.signals.progressSignal.connect(self.showProgress)
            worker.signals.errorSignal.connect(self.showError)
            worker.signals.finishedSignal.connect(self.workerFinished)
            self.workers.append(worker)
            self.pool.start(worker)

    def showError(self, err):
        self.errors.append(err)

    def showProgress(self, fname):
        self.chunkDone += 1
        self.progressSignal.emit(fname, self.chunkDone, self.chunks)

    def workerFinished(self, worker):
        if worker in self.workers:
            self.workers.remove(worker)
        if not self.workers:
            self.stopped = True
            self.finishedSignal.emit(self.errors)
